@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,17 +25,12 @@
 
 #include "gstalsasink.h"
 #include "gstalsasrc.h"
-#include "gstalsamidisrc.h"
-#include "gstalsadeviceprovider.h"
 
 #include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY (alsa_debug);
 
 /* ALSA debugging wrapper */
-/* *INDENT-OFF* */
-G_GNUC_PRINTF (5, 6)
-/* *INDENT-ON* */
 static void
 gst_alsa_error_wrapper (const char *file, int line, const char *function,
     int err, const char *fmt, ...)
@@ -47,7 +42,7 @@ gst_alsa_error_wrapper (const char *file, int line, const char *function,
   va_start (args, fmt);
   str = g_strdup_vprintf (fmt, args);
   va_end (args);
-  /* FIXME: use GST_LEVEL_ERROR here? Currently warning is used because we're
+  /* FIXME: use GST_LEVEL_ERROR here? Currently warning is used because we're 
    * able to catch enough of the errors that would be printed otherwise
    */
   gst_debug_log (alsa_debug, GST_LEVEL_WARNING, file, function, line, NULL,
@@ -68,12 +63,6 @@ plugin_init (GstPlugin * plugin)
   if (!gst_element_register (plugin, "alsasink", GST_RANK_PRIMARY,
           GST_TYPE_ALSA_SINK))
     return FALSE;
-  if (!gst_element_register (plugin, "alsamidisrc", GST_RANK_PRIMARY,
-          GST_TYPE_ALSA_MIDI_SRC))
-    return FALSE;
-  if (!gst_device_provider_register (plugin, "alsadeviceprovider",
-          GST_RANK_SECONDARY, GST_TYPE_ALSA_DEVICE_PROVIDER))
-    return FALSE;
 
   GST_DEBUG_CATEGORY_INIT (alsa_debug, "alsa", 0, "alsa plugins");
 
