@@ -13,33 +13,34 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:gstaudioringbuffer
- * @title: GstAudioRingBuffer
  * @short_description: Base class for audio ringbuffer implementations
  * @see_also: #GstAudioBaseSink, #GstAudioSink
  *
+ * <refsect2>
+ * <para>
  * This object is the base class for audio ringbuffers used by the base
  * audio source and sink classes.
- *
+ * </para>
+ * <para>
  * The ringbuffer abstracts a circular buffer of data. One reader and
  * one writer can operate on the data from different threads in a lockfree
  * manner. The base class is sufficiently flexible to be used as an
  * abstraction for DMA based ringbuffers as well as a pure software
  * implementations.
+ * </para>
+ * </refsect2>
  *
+ * Last reviewed on 2006-02-02 (0.10.4)
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <string.h>
 
-#include <gst/audio/audio.h>
 #include "gstaudioringbuffer.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_audio_ring_buffer_debug);
@@ -81,13 +82,11 @@ gst_audio_ring_buffer_init (GstAudioRingBuffer * ringbuffer)
 {
   ringbuffer->open = FALSE;
   ringbuffer->acquired = FALSE;
-  g_atomic_int_set (&ringbuffer->state, GST_AUDIO_RING_BUFFER_STATE_STOPPED);
+  ringbuffer->state = GST_AUDIO_RING_BUFFER_STATE_STOPPED;
   g_cond_init (&ringbuffer->cond);
   ringbuffer->waiting = 0;
   ringbuffer->empty_seg = NULL;
   ringbuffer->flushing = TRUE;
-  ringbuffer->segbase = 0;
-  ringbuffer->segdone = 0;
 }
 
 static void
@@ -109,9 +108,6 @@ gst_audio_ring_buffer_finalize (GObject * object)
   g_cond_clear (&ringbuffer->cond);
   g_free (ringbuffer->empty_seg);
 
-  if (ringbuffer->cb_data_notify != NULL)
-    ringbuffer->cb_data_notify (ringbuffer->cb_data);
-
   G_OBJECT_CLASS (gst_audio_ring_buffer_parent_class)->finalize (G_OBJECT
       (ringbuffer));
 }
@@ -127,12 +123,7 @@ static const gchar *format_type_names[] = {
   "iec958",
   "ac3",
   "eac3",
-  "dts",
-  "aac mpeg2",
-  "aac mpeg4",
-  "aac mpeg2 raw",
-  "aac mpeg4 raw",
-  "flac"
+  "dts"
 };
 #endif
 
@@ -184,10 +175,9 @@ gst_audio_ring_buffer_debug_spec_buff (GstAudioRingBufferSpec * spec)
   GST_DEBUG ("acquire ringbuffer: total segments: %d", spec->segtotal);
   GST_DEBUG ("acquire ringbuffer: latency segments: %d", spec->seglatency);
   GST_DEBUG ("acquire ringbuffer: segment size: %d bytes = %d samples",
-      spec->segsize, (bpf != 0) ? (spec->segsize / bpf) : -1);
+      spec->segsize, spec->segsize / bpf);
   GST_DEBUG ("acquire ringbuffer: buffer size: %d bytes = %d samples",
-      spec->segsize * spec->segtotal,
-      (bpf != 0) ? (spec->segsize * spec->segtotal / bpf) : -1);
+      spec->segsize * spec->segtotal, spec->segsize * spec->segtotal / bpf);
 }
 
 /**
@@ -224,10 +214,6 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
             gst_structure_get_int (structure, "channels", &info.channels)))
       goto parse_error;
 
-    if (!(gst_audio_channel_positions_from_mask (info.channels, 0,
-                info.position)))
-      goto parse_error;
-
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_A_LAW;
     info.bpf = info.channels;
   } else if (g_str_equal (mimetype, "audio/x-mulaw")) {
@@ -236,10 +222,6 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
             gst_structure_get_int (structure, "channels", &info.channels)))
       goto parse_error;
 
-    if (!(gst_audio_channel_positions_from_mask (info.channels, 0,
-                info.position)))
-      goto parse_error;
-
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MU_LAW;
     info.bpf = info.channels;
   } else if (g_str_equal (mimetype, "audio/x-iec958")) {
@@ -254,7 +236,6 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
     if (!(gst_structure_get_int (structure, "rate", &info.rate)))
       goto parse_error;
 
-    gst_structure_get_int (structure, "channels", &info.channels);
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_AC3;
     info.bpf = 4;
   } else if (g_str_equal (mimetype, "audio/x-eac3")) {
@@ -262,7 +243,6 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
     if (!(gst_structure_get_int (structure, "rate", &info.rate)))
       goto parse_error;
 
-    gst_structure_get_int (structure, "channels", &info.channels);
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_EAC3;
     info.bpf = 16;
   } else if (g_str_equal (mimetype, "audio/x-dts")) {
@@ -270,49 +250,18 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
     if (!(gst_structure_get_int (structure, "rate", &info.rate)))
       goto parse_error;
 
-    gst_structure_get_int (structure, "channels", &info.channels);
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_DTS;
     info.bpf = 4;
   } else if (g_str_equal (mimetype, "audio/mpeg") &&
       gst_structure_get_int (structure, "mpegaudioversion", &i) &&
-      (i == 1 || i == 2 || i == 3)) {
-    /* Now we know this is MPEG-1, MPEG-2 or MPEG-2.5 (non AAC) */
+      (i == 1 || i == 2)) {
+    /* Now we know this is MPEG-1 or MPEG-2 (non AAC) */
     /* extract the needed information from the cap */
     if (!(gst_structure_get_int (structure, "rate", &info.rate)))
       goto parse_error;
 
-    gst_structure_get_int (structure, "channels", &info.channels);
     spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MPEG;
-    info.bpf = 1;
-  } else if (g_str_equal (mimetype, "audio/mpeg") &&
-      gst_structure_get_int (structure, "mpegversion", &i) &&
-      (i == 2 || i == 4) &&
-      (!g_strcmp0 (gst_structure_get_string (structure, "stream-format"),
-              "adts")
-          || !g_strcmp0 (gst_structure_get_string (structure, "stream-format"),
-              "raw"))) {
-    /* MPEG-2 AAC or MPEG-4 AAC */
-    if (!(gst_structure_get_int (structure, "rate", &info.rate)))
-      goto parse_error;
-
-    gst_structure_get_int (structure, "channels", &info.channels);
-    if (!g_strcmp0 (gst_structure_get_string (structure, "stream-format"),
-            "adts"))
-      spec->type = (i == 2) ? GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MPEG2_AAC :
-          GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MPEG4_AAC;
-    else
-      spec->type = (i == 2) ?
-          GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MPEG2_AAC_RAW :
-          GST_AUDIO_RING_BUFFER_FORMAT_TYPE_MPEG4_AAC_RAW;
-    info.bpf = 1;
-  } else if (g_str_equal (mimetype, "audio/x-flac")) {
-    /* extract the needed information from the cap */
-    if (!(gst_structure_get_int (structure, "rate", &info.rate)))
-      goto parse_error;
-
-    gst_structure_get_int (structure, "channels", &info.channels);
-    spec->type = GST_AUDIO_RING_BUFFER_FORMAT_TYPE_FLAC;
-    info.bpf = 1;
+    info.bpf = 4;
   } else {
     goto parse_error;
   }
@@ -355,7 +304,7 @@ parse_error:
  * @src_fmt: the source format
  * @src_val: the source value
  * @dest_fmt: the destination format
- * @dest_val: (out): a location to store the converted value
+ * @dest_val: a location to store the converted value
  *
  * Convert @src_val in @src_fmt to the equivalent value in @dest_fmt. The result
  * will be put in @dest_val.
@@ -378,9 +327,9 @@ gst_audio_ring_buffer_convert (GstAudioRingBuffer * buf,
 }
 
 /**
- * gst_audio_ring_buffer_set_callback: (skip)
+ * gst_audio_ring_buffer_set_callback:
  * @buf: the #GstAudioRingBuffer to set the callback on
- * @cb: (allow-none): the callback to set
+ * @cb: (scope async): the callback to set
  * @user_data: user data passed to the callback
  *
  * Sets the given callback function on the buffer. This function
@@ -392,44 +341,12 @@ void
 gst_audio_ring_buffer_set_callback (GstAudioRingBuffer * buf,
     GstAudioRingBufferCallback cb, gpointer user_data)
 {
-  gst_audio_ring_buffer_set_callback_full (buf, cb, user_data, NULL);
-}
-
-/**
- * gst_audio_ring_buffer_set_callback_full: (rename-to gst_audio_ring_buffer_set_callback)
- * @buf: the #GstAudioRingBuffer to set the callback on
- * @cb: (allow-none): the callback to set
- * @user_data: user data passed to the callback
- * @notify: function to be called when @user_data is no longer needed
- *
- * Sets the given callback function on the buffer. This function
- * will be called every time a segment has been written to a device.
- *
- * MT safe.
- *
- * Since: 1.12
- */
-void
-gst_audio_ring_buffer_set_callback_full (GstAudioRingBuffer * buf,
-    GstAudioRingBufferCallback cb, gpointer user_data, GDestroyNotify notify)
-{
-  gpointer old_data = NULL;
-  GDestroyNotify old_notify;
-
   g_return_if_fail (GST_IS_AUDIO_RING_BUFFER (buf));
 
   GST_OBJECT_LOCK (buf);
-  old_notify = buf->cb_data_notify;
-  old_data = buf->cb_data;
-
   buf->callback = cb;
   buf->cb_data = user_data;
-  buf->cb_data_notify = notify;
   GST_OBJECT_UNLOCK (buf);
-
-  if (old_notify) {
-    old_notify (old_data);
-  }
 }
 
 
@@ -656,7 +573,7 @@ gst_audio_ring_buffer_acquire (GstAudioRingBuffer * buf,
   buf->empty_seg = g_malloc (segsize);
 
   if (buf->spec.type == GST_AUDIO_RING_BUFFER_FORMAT_TYPE_RAW) {
-    gst_audio_format_info_fill_silence (buf->spec.info.finfo, buf->empty_seg,
+    gst_audio_format_fill_silence (buf->spec.info.finfo, buf->empty_seg,
         segsize);
   } else {
     /* FIXME, non-raw formats get 0 as the empty sample */
@@ -737,7 +654,7 @@ gst_audio_ring_buffer_release (GstAudioRingBuffer * buf)
   buf->acquired = FALSE;
 
   /* if this fails, something is wrong in this file */
-  g_assert (buf->open);
+  g_assert (buf->open == TRUE);
 
   rclass = GST_AUDIO_RING_BUFFER_GET_CLASS (buf);
   if (G_LIKELY (rclass->release))
@@ -750,8 +667,6 @@ gst_audio_ring_buffer_release (GstAudioRingBuffer * buf)
   if (G_UNLIKELY (!res))
     goto release_failed;
 
-  g_atomic_int_set (&buf->segdone, 0);
-  buf->segbase = 0;
   g_free (buf->empty_seg);
   buf->empty_seg = NULL;
   gst_caps_replace (&buf->spec.caps, NULL);
@@ -972,7 +887,7 @@ gst_audio_ring_buffer_start (GstAudioRingBuffer * buf)
   if (G_UNLIKELY (!buf->acquired))
     goto not_acquired;
 
-  if (G_UNLIKELY (!g_atomic_int_get (&buf->may_start)))
+  if (G_UNLIKELY (g_atomic_int_get (&buf->may_start) == FALSE))
     goto may_not_start;
 
   /* if stopped, set to started */
@@ -1005,7 +920,7 @@ gst_audio_ring_buffer_start (GstAudioRingBuffer * buf)
   }
 
   if (G_UNLIKELY (!res)) {
-    g_atomic_int_set (&buf->state, GST_AUDIO_RING_BUFFER_STATE_PAUSED);
+    buf->state = GST_AUDIO_RING_BUFFER_STATE_PAUSED;
     GST_DEBUG_OBJECT (buf, "failed to start");
   } else {
     GST_DEBUG_OBJECT (buf, "started");
@@ -1036,40 +951,6 @@ may_not_start:
   }
 }
 
-G_GNUC_INTERNAL
-    void __gst_audio_ring_buffer_set_errored (GstAudioRingBuffer * buf);
-
-/* __gst_audio_ring_buffer_set_errored:
- * @buf: the #GstAudioRingBuffer that has encountered an error
- *
- * Mark the ringbuffer as errored after it has started.
- *
- * MT safe.
-
- * Since: 1.24 (internal in 1.22)
- */
-void
-__gst_audio_ring_buffer_set_errored (GstAudioRingBuffer * buf)
-{
-  gboolean res;
-
-  /* If started set to errored */
-  res = g_atomic_int_compare_and_exchange (&buf->state,
-      GST_AUDIO_RING_BUFFER_STATE_STARTED, GST_AUDIO_RING_BUFFER_STATE_ERROR);
-  if (!res) {
-    GST_DEBUG_OBJECT (buf, "ringbuffer was not started, checking paused");
-    res = g_atomic_int_compare_and_exchange (&buf->state,
-        GST_AUDIO_RING_BUFFER_STATE_PAUSED, GST_AUDIO_RING_BUFFER_STATE_ERROR);
-  }
-  if (res) {
-    GST_DEBUG_OBJECT (buf, "ringbuffer is errored");
-  } else {
-    GST_DEBUG_OBJECT (buf,
-        "Could not mark ringbuffer as errored. It must have been stopped or already errored (was state %d)",
-        g_atomic_int_get (&buf->state));
-  }
-}
-
 static gboolean
 gst_audio_ring_buffer_pause_unlocked (GstAudioRingBuffer * buf)
 {
@@ -1094,8 +975,7 @@ gst_audio_ring_buffer_pause_unlocked (GstAudioRingBuffer * buf)
     res = rclass->pause (buf);
 
   if (G_UNLIKELY (!res)) {
-    /* Restore started state */
-    g_atomic_int_set (&buf->state, GST_AUDIO_RING_BUFFER_STATE_STARTED);
+    buf->state = GST_AUDIO_RING_BUFFER_STATE_STARTED;
     GST_DEBUG_OBJECT (buf, "failed to pause");
   } else {
     GST_DEBUG_OBJECT (buf, "paused");
@@ -1106,7 +986,7 @@ gst_audio_ring_buffer_pause_unlocked (GstAudioRingBuffer * buf)
 not_started:
   {
     /* was not started */
-    GST_DEBUG_OBJECT (buf, "was not started (state %d)", buf->state);
+    GST_DEBUG_OBJECT (buf, "was not started");
     return TRUE;
   }
 }
@@ -1188,16 +1068,9 @@ gst_audio_ring_buffer_stop (GstAudioRingBuffer * buf)
         GST_AUDIO_RING_BUFFER_STATE_PAUSED,
         GST_AUDIO_RING_BUFFER_STATE_STOPPED);
     if (!res) {
-      GST_DEBUG_OBJECT (buf, "was not paused, try errored");
-      res = g_atomic_int_compare_and_exchange (&buf->state,
-          GST_AUDIO_RING_BUFFER_STATE_ERROR,
-          GST_AUDIO_RING_BUFFER_STATE_STOPPED);
-    }
-    if (!res) {
-      /* was not paused or stopped either, must have been stopped then */
+      /* was not paused either, must have been stopped then */
       res = TRUE;
-      GST_DEBUG_OBJECT (buf,
-          "was not paused or errored, must have been stopped");
+      GST_DEBUG_OBJECT (buf, "was not paused, must have been stopped");
       goto done;
     }
   }
@@ -1211,7 +1084,7 @@ gst_audio_ring_buffer_stop (GstAudioRingBuffer * buf)
     res = rclass->stop (buf);
 
   if (G_UNLIKELY (!res)) {
-    g_atomic_int_set (&buf->state, GST_AUDIO_RING_BUFFER_STATE_STARTED);
+    buf->state = GST_AUDIO_RING_BUFFER_STATE_STARTED;
     GST_DEBUG_OBJECT (buf, "failed to stop");
   } else {
     GST_DEBUG_OBJECT (buf, "stopped");
@@ -1231,11 +1104,11 @@ done:
  * implementation uses another internal buffer between the audio
  * device.
  *
- * For playback ringbuffers this is the amount of samples transferred from the
+ * For playback ringbuffers this is the amount of samples transfered from the
  * ringbuffer to the device but still not played.
  *
  * For capture ringbuffers this is the amount of samples in the device that are
- * not yet transferred to the ringbuffer.
+ * not yet transfered to the ringbuffer.
  *
  * Returns: The number of samples queued in the audio device.
  *
@@ -1322,7 +1195,7 @@ gst_audio_ring_buffer_set_sample (GstAudioRingBuffer * buf, guint64 sample)
   if (G_UNLIKELY (buf->samples_per_seg == 0))
     return;
 
-  /* FIXME, we assume the ringbuffer can restart at a random
+  /* FIXME, we assume the ringbuffer can restart at a random 
    * position, round down to the beginning and keep track of
    * offset when calculating the processed samples. */
   buf->segbase = buf->segdone - sample / buf->samples_per_seg;
@@ -1333,12 +1206,6 @@ gst_audio_ring_buffer_set_sample (GstAudioRingBuffer * buf, guint64 sample)
       sample, buf->segbase);
 }
 
-/**
- * default_clear_all:
- * @buf: the #GstAudioRingBuffer to clear
- *
- * Fill the ringbuffer with silence.
- */
 static void
 default_clear_all (GstAudioRingBuffer * buf)
 {
@@ -1359,7 +1226,7 @@ default_clear_all (GstAudioRingBuffer * buf)
  * gst_audio_ring_buffer_clear_all:
  * @buf: the #GstAudioRingBuffer to clear
  *
- * Clear all samples from the ringbuffer.
+ * Fill the ringbuffer with silence.
  *
  * MT safe.
  */
@@ -1387,7 +1254,7 @@ wait_segment (GstAudioRingBuffer * buf)
   if (G_UNLIKELY (g_atomic_int_get (&buf->state) !=
           GST_AUDIO_RING_BUFFER_STATE_STARTED)) {
     /* see if we are allowed to start it */
-    if (G_UNLIKELY (!g_atomic_int_get (&buf->may_start)))
+    if (G_UNLIKELY (g_atomic_int_get (&buf->may_start) == FALSE))
       goto no_start;
 
     GST_DEBUG_OBJECT (buf, "start!");
@@ -1602,8 +1469,6 @@ default_commit (GstAudioRingBuffer * buf, guint64 * sample,
   writeseg = *sample / sps;
   sampleoff = (*sample % sps) * bpf;
 
-  GST_DEBUG_OBJECT (buf, "write %d : %d", in_samples, out_samples);
-
   /* write out all samples */
   while (*toprocess > 0) {
     gint avail;
@@ -1620,13 +1485,14 @@ default_commit (GstAudioRingBuffer * buf, guint64 * sample,
       /* see how far away it is from the write segment */
       diff = writeseg - segdone;
 
-      GST_DEBUG_OBJECT (buf,
-          "pointer at %d, write to %d-%d, diff %d, segtotal %d, segsize %d, base %d",
+      GST_DEBUG
+          ("pointer at %d, write to %d-%d, diff %d, segtotal %d, segsize %d, base %d",
           segdone, writeseg, sampleoff, diff, segtotal, segsize, buf->segbase);
 
       /* segment too far ahead, writer too slow, we need to drop, hopefully UNLIKELY */
       if (G_UNLIKELY (diff < 0)) {
-        /* we need to drop one segment at a time, pretend we wrote a segment. */
+        /* we need to drop one segment at a time, pretend we wrote a
+         * segment. */
         skip = TRUE;
         break;
       }
@@ -1717,11 +1583,11 @@ not_started:
 /**
  * gst_audio_ring_buffer_commit:
  * @buf: the #GstAudioRingBuffer to commit
- * @sample: (inout): the sample position of the data
- * @data: (array length=in_samples): the data to commit
+ * @sample: the sample position of the data
+ * @data: the data to commit
  * @in_samples: the number of samples in the data to commit
  * @out_samples: the number of samples to write to the ringbuffer
- * @accum: (inout): accumulator for rate conversion.
+ * @accum: accumulator for rate conversion.
  *
  * Commit @in_samples samples pointed to by @data to the ringbuffer @buf.
  *
@@ -1771,11 +1637,11 @@ gst_audio_ring_buffer_commit (GstAudioRingBuffer * buf, guint64 * sample,
  * gst_audio_ring_buffer_read:
  * @buf: the #GstAudioRingBuffer to read from
  * @sample: the sample position of the data
- * @data: (array length=len): where the data should be read
+ * @data: where the data should be read
  * @len: the number of samples in data to read
- * @timestamp: (out): where the timestamp is returned
+ * @timestamp: where the timestamp is returned
  *
- * Read @len samples from the ringbuffer into the memory pointed
+ * Read @len samples from the ringbuffer into the memory pointed 
  * to by @data.
  * The first sample should be read from position @sample in
  * the ringbuffer.
@@ -1835,8 +1701,8 @@ gst_audio_ring_buffer_read (GstAudioRingBuffer * buf, guint64 sample,
        * reading) */
       diff = segdone - readseg;
 
-      GST_DEBUG_OBJECT
-          (buf, "pointer at %d, sample %" G_GUINT64_FORMAT
+      GST_DEBUG
+          ("pointer at %d, sample %" G_GUINT64_FORMAT
           ", read from %d-%d, to_read %d, diff %d, segtotal %d, segsize %d",
           segdone, sample, readseg, sampleoff, to_read, diff, segtotal,
           segsize);
@@ -1874,7 +1740,7 @@ gst_audio_ring_buffer_read (GstAudioRingBuffer * buf, guint64 sample,
       /* Reorder from device order to GStreamer order */
       for (i = 0; i < sampleslen; i++) {
         for (j = 0; j < channels; j++) {
-          memcpy (data + i * bpf + reorder_map[j] * bps, ptr + j * bps, bps);
+          memcpy (data + reorder_map[j] * bps, ptr + j * bps, bps);
         }
         ptr += bpf;
       }
@@ -1891,7 +1757,7 @@ gst_audio_ring_buffer_read (GstAudioRingBuffer * buf, guint64 sample,
 
   if (buf->timestamps && timestamp) {
     *timestamp = buf->timestamps[readseg % segtotal];
-    GST_DEBUG_OBJECT (buf, "Retrieved timestamp %" GST_TIME_FORMAT
+    GST_INFO_OBJECT (buf, "Retrieved timestamp %" GST_TIME_FORMAT
         " @ %d", GST_TIME_ARGS (*timestamp), readseg % segtotal);
   }
 
@@ -1908,10 +1774,9 @@ not_started:
 /**
  * gst_audio_ring_buffer_prepare_read:
  * @buf: the #GstAudioRingBuffer to read from
- * @segment: (out): the segment to read
- * @readptr: (out) (array length=len):
- *     the pointer to the memory where samples can be read
- * @len: (out): the number of bytes to read
+ * @segment: the segment to read
+ * @readptr: the pointer to the memory where samples can be read
+ * @len: the number of bytes to read
  *
  * Returns a pointer to memory where the data from segment @segment
  * can be found. This function is mostly used by subclasses.
@@ -1949,7 +1814,7 @@ gst_audio_ring_buffer_prepare_read (GstAudioRingBuffer * buf, gint * segment,
   *len = buf->spec.segsize;
   *readptr = data + *segment * *len;
 
-  GST_LOG_OBJECT (buf, "prepare read from segment %d (real %d) @%p",
+  GST_LOG ("prepare read from segment %d (real %d) @%p",
       *segment, segdone, *readptr);
 
   /* callback to fill the memory with data, for pull based
@@ -1965,7 +1830,7 @@ gst_audio_ring_buffer_prepare_read (GstAudioRingBuffer * buf, gint * segment,
  * @buf: the #GstAudioRingBuffer to advance
  * @advance: the number of segments written
  *
- * Subclasses should call this function to notify the fact that
+ * Subclasses should call this function to notify the fact that 
  * @advance segments are now processed by the device.
  *
  * MT safe.
@@ -2019,7 +1884,7 @@ gst_audio_ring_buffer_clear (GstAudioRingBuffer * buf, gint segment)
   data = buf->memory;
   data += segment * buf->spec.segsize;
 
-  GST_LOG_OBJECT (buf, "clear segment %d @%p", segment, data);
+  GST_LOG ("clear segment %d @%p", segment, data);
 
   memcpy (data, buf->empty_seg, buf->spec.segsize);
 }
@@ -2030,7 +1895,7 @@ gst_audio_ring_buffer_clear (GstAudioRingBuffer * buf, gint segment)
  * @allowed: the new value
  *
  * Tell the ringbuffer that it is allowed to start playback when
- * the ringbuffer is filled with samples.
+ * the ringbuffer is filled with samples. 
  *
  * MT safe.
  */
@@ -2043,27 +1908,10 @@ gst_audio_ring_buffer_may_start (GstAudioRingBuffer * buf, gboolean allowed)
   g_atomic_int_set (&buf->may_start, allowed);
 }
 
-/* GST_AUDIO_CHANNEL_POSITION_NONE is used for position-less
- * mutually exclusive channels. In this case we should not attempt
- * to do any reordering.
- */
-static gboolean
-position_less_channels (const GstAudioChannelPosition * pos, guint channels)
-{
-  guint i;
-
-  for (i = 0; i < channels; i++) {
-    if (pos[i] != GST_AUDIO_CHANNEL_POSITION_NONE)
-      return FALSE;
-  }
-
-  return TRUE;
-}
-
 /**
  * gst_audio_ring_buffer_set_channel_positions:
  * @buf: the #GstAudioRingBuffer
- * @position: (array): the device channel positions
+ * @position: the device channel positions
  *
  * Tell the ringbuffer about the device's channel positions. This must
  * be called in when the ringbuffer is acquired.
@@ -2086,30 +1934,12 @@ gst_audio_ring_buffer_set_channel_positions (GstAudioRingBuffer * buf,
   if (memcmp (position, to, channels * sizeof (to[0])) == 0)
     return;
 
-  if (position_less_channels (position, channels)) {
-    GST_LOG_OBJECT (buf, "position-less channels, no need to reorder");
-    return;
-  }
-
   if (!gst_audio_get_channel_reorder_map (channels, position, to,
           buf->channel_reorder_map))
     g_return_if_reached ();
 
   for (i = 0; i < channels; i++) {
     if (buf->channel_reorder_map[i] != i) {
-#ifndef GST_DISABLE_GST_DEBUG
-      {
-        gchar *tmp1, *tmp2;
-
-        tmp1 = gst_audio_channel_positions_to_string (position, channels);
-        tmp2 = gst_audio_channel_positions_to_string (to, channels);
-        GST_LOG_OBJECT (buf, "may have to reorder channels: %s -> %s", tmp1,
-            tmp2);
-        g_free (tmp1);
-        g_free (tmp2);
-      }
-#endif /* GST_DISABLE_GST_DEBUG */
-
       buf->need_reorder = TRUE;
       break;
     }
@@ -2125,6 +1955,8 @@ gst_audio_ring_buffer_set_channel_positions (GstAudioRingBuffer * buf,
  * Set a new timestamp on the buffer.
  *
  * MT safe.
+ *
+ * Since:
  */
 void
 gst_audio_ring_buffer_set_timestamp (GstAudioRingBuffer * buf, gint readseg,
@@ -2132,22 +1964,11 @@ gst_audio_ring_buffer_set_timestamp (GstAudioRingBuffer * buf, gint readseg,
 {
   g_return_if_fail (GST_IS_AUDIO_RING_BUFFER (buf));
 
-  GST_DEBUG_OBJECT (buf, "Storing timestamp %" GST_TIME_FORMAT
+  GST_INFO_OBJECT (buf, "Storing timestamp %" GST_TIME_FORMAT
       " @ %d", GST_TIME_ARGS (timestamp), readseg);
-
-  GST_OBJECT_LOCK (buf);
-  if (G_UNLIKELY (!buf->acquired))
-    goto not_acquired;
-
-  buf->timestamps[readseg] = timestamp;
-
-done:
-  GST_OBJECT_UNLOCK (buf);
-  return;
-
-not_acquired:
-  {
-    GST_DEBUG_OBJECT (buf, "we are not acquired");
-    goto done;
+  if (buf->timestamps) {
+    buf->timestamps[readseg] = timestamp;
+  } else {
+    GST_ERROR_OBJECT (buf, "Could not store timestamp, no timestamps buffer");
   }
 }
