@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,7 +50,7 @@ G_DEFINE_TYPE (GstDiscovererStreamInfo, gst_discoverer_stream_info,
 static void
 gst_discoverer_stream_info_init (GstDiscovererStreamInfo * info)
 {
-  info->stream_number = -1;
+  /* Nothing needs initialization */
 }
 
 static void
@@ -70,8 +70,6 @@ gst_discoverer_stream_info_finalize (GObject * object)
   if (info->toc)
     gst_toc_unref (info->toc);
 
-  g_free (info->stream_id);
-
   if (info->misc)
     gst_structure_free (info->misc);
 }
@@ -134,17 +132,12 @@ gst_discoverer_info_copy_int (GstDiscovererStreamInfo * info,
   if (info->toc)
     ret->toc = gst_toc_ref (info->toc);
 
-  if (info->stream_id)
-    ret->stream_id = g_strdup (info->stream_id);
-
   if (info->misc)
     ret->misc = gst_structure_copy (info->misc);
 
   if (stream_map)
     g_hash_table_insert (stream_map, info, ret);
 
-  ret->stream_number = info->stream_number;
-
   return ret;
 }
 
@@ -177,9 +170,6 @@ gst_discoverer_container_info_finalize (GObject * object)
 
   gst_discoverer_stream_info_list_free (info->streams);
 
-  if (info->tags)
-    gst_tag_list_unref (info->tags);
-
   gst_discoverer_stream_info_finalize ((GObject *) info);
 }
 
@@ -209,9 +199,6 @@ gst_stream_container_info_copy_int (GstDiscovererContainerInfo * ptr,
       g_hash_table_insert (stream_map, tmp->data, subtop);
   }
 
-  if (ptr->tags)
-    ret->tags = gst_tag_list_copy (ptr->tags);
-
   return ret;
 }
 
@@ -256,7 +243,6 @@ gst_discoverer_audio_info_copy_int (GstDiscovererAudioInfo * ptr)
   ret = gst_discoverer_audio_info_new ();
 
   ret->channels = ptr->channels;
-  ret->channel_mask = ptr->channel_mask;
   ret->sample_rate = ptr->sample_rate;
   ret->depth = ptr->depth;
   ret->bitrate = ptr->bitrate;
@@ -362,7 +348,7 @@ G_DEFINE_TYPE (GstDiscovererInfo, gst_discoverer_info, G_TYPE_OBJECT);
 static void
 gst_discoverer_info_init (GstDiscovererInfo * info)
 {
-  info->missing_elements_details = g_ptr_array_new_with_free_func (g_free);
+  /* Nothing needs initialization */
 }
 
 static void
@@ -384,10 +370,6 @@ gst_discoverer_info_finalize (GObject * object)
 
   if (info->toc)
     gst_toc_unref (info->toc);
-
-  g_free (info->cachefile);
-
-  g_ptr_array_unref (info->missing_elements_details);
 }
 
 static GstDiscovererInfo *
@@ -421,9 +403,6 @@ gst_discoverer_info_copy (GstDiscovererInfo * ptr)
         stream_map);
   }
   ret->duration = ptr->duration;
-  ret->result = ptr->result;
-  ret->seekable = ptr->seekable;
-  ret->live = ptr->live;
   if (ptr->misc)
     ret->misc = gst_structure_copy (ptr->misc);
 
@@ -684,29 +663,10 @@ gst_discoverer_stream_info_get_toc (GstDiscovererStreamInfo * info)
   return info->toc;
 }
 
-/**
- * gst_discoverer_stream_info_get_stream_id:
- * @info: a #GstDiscovererStreamInfo
- *
- * Returns: (transfer none): the stream ID of this stream. If you wish to
- * use the stream ID after the life-time of @info you will need to copy it.
- */
-const gchar *
-gst_discoverer_stream_info_get_stream_id (GstDiscovererStreamInfo * info)
-{
-  g_return_val_if_fail (GST_IS_DISCOVERER_STREAM_INFO (info), NULL);
-
-  return info->stream_id;
-}
-
-#ifndef GST_REMOVE_DEPRECATED
 /**
  * gst_discoverer_stream_info_get_misc:
  * @info: a #GstDiscovererStreamInfo
  *
- * Deprecated: This functions is deprecated since version 1.4, use
- * #gst_discoverer_info_get_missing_elements_installer_details
- *
  * Returns: (transfer none): additional information regarding the stream (for
  * example codec version, profile, etc..). If you wish to use the #GstStructure
  * after the life-time of @info you will need to copy it.
@@ -718,24 +678,6 @@ gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
 
   return info->misc;
 }
-#endif
-
-/**
- * gst_discoverer_stream_info_get_stream_number:
- * @info: a #GstDiscovererStreamInfo
- *
- * Returns: the stream number, -1 if no index could be determined. This property
- * acts as a unique identifier as a 'int' for the stream.
- *
- * Since: 1.20
- */
-gint
-gst_discoverer_stream_info_get_stream_number (GstDiscovererStreamInfo * info)
-{
-  g_return_val_if_fail (GST_IS_DISCOVERER_STREAM_INFO (info), -1);
-
-  return info->stream_number;
-}
 
 /* GstDiscovererContainerInfo */
 
@@ -763,23 +705,6 @@ gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
   return res;
 }
 
-/**
- * gst_discoverer_container_info_get_tags:
- * @info: a #GstDiscovererStreamInfo
- *
- * Returns: (transfer none): tags specific to the given container. If you wish to use
- * the tags after the life-time of @info, you will need to copy them.
- *
- * Since: 1.20
- */
-const GstTagList *
-gst_discoverer_container_info_get_tags (const GstDiscovererContainerInfo * info)
-{
-  g_return_val_if_fail (GST_IS_DISCOVERER_CONTAINER_INFO (info), NULL);
-
-  return info->tags;
-}
-
 /* GstDiscovererAudioInfo */
 
 #define AUDIO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
@@ -796,19 +721,6 @@ gst_discoverer_container_info_get_tags (const GstDiscovererContainerInfo * info)
 
 AUDIO_INFO_ACCESSOR_CODE (channels, guint, 0);
 
-/**
- * gst_discoverer_audio_info_get_channel_mask:
- * @info: a #GstDiscovererAudioInfo
- *
- * Returns: the channel-mask of the stream, refer to
- * gst_audio_channel_positions_from_mask() for more
- * information.
- *
- * Since: 1.14
- */
-
-AUDIO_INFO_ACCESSOR_CODE (channel_mask, guint64, G_MAXUINT64);
-
 /**
  * gst_discoverer_audio_info_get_sample_rate:
  * @info: a #GstDiscovererAudioInfo
@@ -960,7 +872,7 @@ VIDEO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
  * gst_discoverer_video_info_is_image:
  * @info: a #GstDiscovererVideoInfo
  *
- * Returns: %TRUE if the video stream corresponds to an image (i.e. only contains
+ * Returns: #TRUE if the video stream corresponds to an image (i.e. only contains
  * one frame).
  */
 gboolean
@@ -1074,25 +986,10 @@ DISCOVERER_INFO_ACCESSOR_CODE (duration, GstClockTime, GST_CLOCK_TIME_NONE);
 
 DISCOVERER_INFO_ACCESSOR_CODE (seekable, gboolean, FALSE);
 
-/**
- * gst_discoverer_info_get_live:
- * @info: a #GstDiscovererInfo
- *
- * Returns: whether the URI is live.
- *
- * Since: 1.14
- */
-
-DISCOVERER_INFO_ACCESSOR_CODE (live, gboolean, FALSE);
-
-#ifndef GST_REMOVE_DEPRECATED
 /**
  * gst_discoverer_info_get_misc:
  * @info: a #GstDiscovererInfo
  *
- * Deprecated: This functions is deprecated since version 1.4, use
- * #gst_discoverer_info_get_missing_elements_installer_details
- *
  * Returns: (transfer none): Miscellaneous information stored as a #GstStructure
  * (for example: information about missing plugins). If you wish to use the
  * #GstStructure after the life-time of @info, you will need to copy it.
@@ -1106,12 +1003,9 @@ DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
  *
  * Returns: (transfer none): all tags contained in the URI. If you wish to use
  * the tags after the life-time of @info, you will need to copy them.
- *
- * Deprecated: 1.20: Use gst_discoverer_{container,stream}_info_get_tags() instead.
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (tags, const GstTagList *, NULL);
-#endif
 
 /**
  * gst_discoverer_info_get_toc:
@@ -1154,39 +1048,3 @@ DISCOVERER_INFO_ACCESSOR_CODE (toc, const GstToc *, NULL);
  *
  * Decrements the reference count of @info.
  */
-
-
-/**
- * gst_discoverer_info_get_missing_elements_installer_details:
- * @info: a #GstDiscovererStreamInfo to retrieve installer detail
- * for the missing element
- *
- * Get the installer details for missing elements
- *
- * Returns: (transfer none) (array zero-terminated=1): An array of strings
- * containing information about how to install the various missing elements
- * for @info to be usable. If you wish to use the strings after the life-time
- * of @info, you will need to copy them.
- *
- * Since: 1.4
- */
-const gchar **
-gst_discoverer_info_get_missing_elements_installer_details (const
-    GstDiscovererInfo * info)
-{
-
-  if (info->result != GST_DISCOVERER_MISSING_PLUGINS) {
-    GST_WARNING_OBJECT (info, "Trying to get missing element installed details "
-        "but result is not 'MISSING_PLUGINS'");
-
-    return NULL;
-  }
-
-  if (info->missing_elements_details->pdata[info->missing_elements_details->
-          len]) {
-    GST_DEBUG ("Adding NULL pointer to the end of missing_elements_details");
-    g_ptr_array_add (info->missing_elements_details, NULL);
-  }
-
-  return (const gchar **) info->missing_elements_details->pdata;
-}
