@@ -14,24 +14,25 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:gstrtpbuffer
- * @title: GstRTPBuffer
  * @short_description: Helper methods for dealing with RTP buffers
  * @see_also: #GstRTPBasePayload, #GstRTPBaseDepayload, gstrtcpbuffer
  *
- * The GstRTPBuffer helper functions makes it easy to parse and create regular
+ * <refsect2>
+ * <para>
+ * The GstRTPBuffer helper functions makes it easy to parse and create regular 
  * #GstBuffer objects that contain RTP payloads. These buffers are typically of
  * 'application/x-rtp' #GstCaps.
+ * </para>
+ * </refsect2>
  *
+ * Last reviewed on 2006-07-17 (0.10.10)
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "gstrtpbuffer.h"
 
@@ -105,6 +106,7 @@ gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
 
   g_return_if_fail (csrc_count <= 15);
   g_return_if_fail (GST_IS_BUFFER (buffer));
+  g_return_if_fail (pad_len <= 255);
   g_return_if_fail (gst_buffer_is_writable (buffer));
 
   gst_buffer_remove_all_memory (buffer);
@@ -158,7 +160,7 @@ gst_rtp_buffer_allocate_data (GstBuffer * buffer, guint payload_len,
  * respectively. @data will be freed when the buffer is unreffed, so this
  * function transfers ownership of @data to the new buffer.
  *
- * Returns: (transfer full): A newly allocated buffer with @data and of size @len.
+ * Returns: A newly allocated buffer with @data and of size @len.
  */
 GstBuffer *
 gst_rtp_buffer_new_take_data (gpointer data, gsize len)
@@ -179,12 +181,12 @@ gst_rtp_buffer_new_take_data (gpointer data, gsize len)
  * bytes of @data and the size to @len. The data will be freed when the buffer
  * is freed.
  *
- * Returns: (transfer full): A newly allocated buffer with a copy of @data and of size @len.
+ * Returns: A newly allocated buffer with a copy of @data and of size @len.
  */
 GstBuffer *
-gst_rtp_buffer_new_copy_data (gconstpointer data, gsize len)
+gst_rtp_buffer_new_copy_data (gpointer data, gsize len)
 {
-  return gst_rtp_buffer_new_take_data (g_memdup2 (data, len), len);
+  return gst_rtp_buffer_new_take_data (g_memdup (data, len), len);
 }
 
 /**
@@ -197,7 +199,7 @@ gst_rtp_buffer_new_copy_data (gconstpointer data, gsize len)
  * @csrc_count CSRCs, a payload length of @payload_len and padding of @pad_len.
  * All other RTP header fields will be set to 0/FALSE.
  *
- * Returns: (transfer full): A newly allocated buffer that can hold an RTP packet with given
+ * Returns: A newly allocated buffer that can hold an RTP packet with given
  * parameters.
  */
 GstBuffer *
@@ -225,7 +227,7 @@ gst_rtp_buffer_new_allocate (guint payload_len, guint8 pad_len,
  * @csrc_count and can be calculated with gst_rtp_buffer_calc_payload_len().
  * All RTP header fields will be set to 0/FALSE.
  *
- * Returns: (transfer full): A newly allocated buffer that can hold an RTP packet of @packet_len.
+ * Returns: A newly allocated buffer that can hold an RTP packet of @packet_len.
  */
 GstBuffer *
 gst_rtp_buffer_new_allocate_len (guint packet_len, guint8 pad_len,
@@ -295,10 +297,6 @@ gst_rtp_buffer_calc_payload_len (guint packet_len, guint8 pad_len,
 {
   g_return_val_if_fail (csrc_count <= 15, 0);
 
-  if (packet_len <
-      GST_RTP_HEADER_LEN + (csrc_count * sizeof (guint32)) + pad_len)
-    return 0;
-
   return packet_len - GST_RTP_HEADER_LEN - (csrc_count * sizeof (guint32))
       - pad_len;
 }
@@ -319,19 +317,17 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
   guint8 padding;
   guint8 csrc_count;
   guint header_len;
-  guint8 version, pt;
+  guint8 version;
   guint8 *data;
   guint size;
   gsize bufsize, skip;
   guint idx, length;
-  guint n_mem;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
   g_return_val_if_fail (rtp != NULL, FALSE);
   g_return_val_if_fail (rtp->buffer == NULL, FALSE);
 
-  n_mem = gst_buffer_n_memory (buffer);
-  if (n_mem < 1)
+  if (gst_buffer_n_memory (buffer) < 1)
     goto no_memory;
 
   /* map first memory, this should be the header */
@@ -351,13 +347,6 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
   if (G_UNLIKELY (version != (GST_RTP_VERSION << 6)))
     goto wrong_version;
 
-  /* check reserved PT and marker bit, this is to check for RTCP
-   * packets. We do a relaxed check, you can still use 72-76 as long
-   * as the marker bit is cleared. */
-  pt = data[1];
-  if (G_UNLIKELY (pt >= 200 && pt <= 204))
-    goto reserved_pt;
-
   /* calc header length with csrc */
   csrc_count = (data[0] & 0x0f);
   header_len += csrc_count * sizeof (guint32);
@@ -369,7 +358,7 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
   /* calc extension length when present. */
   if (data[0] & 0x10) {
     guint8 *extdata;
-    gsize extlen;
+    guint16 extlen;
 
     /* find memory for the extension bits, we find the block for the first 4
      * bytes, all other extension bytes should also be in this block */
@@ -400,9 +389,8 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
     rtp->size[1] = 0;
   }
 
-  /* check for padding unless flags says to skip */
-  if ((data[0] & 0x20) != 0 &&
-      (flags & GST_RTP_BUFFER_MAP_FLAG_SKIP_PADDING) == 0) {
+  /* check for padding */
+  if (data[0] & 0x20) {
     /* find memory for the padding bits */
     if (!gst_buffer_find_memory (buffer, bufsize - 1, 1, &idx, &length, &skip))
       goto wrong_length;
@@ -411,11 +399,11 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
       goto map_failed;
 
     padding = rtp->map[3].data[skip];
-    rtp->data[3] = rtp->map[3].data + skip + 1 - padding;
-    rtp->size[3] = padding;
-
     if (skip + 1 < padding)
       goto wrong_length;
+
+    rtp->data[3] = rtp->map[3].data + skip + 1 - padding;
+    rtp->size[3] = padding;
   } else {
     rtp->data[3] = NULL;
     rtp->size[3] = 0;
@@ -427,19 +415,10 @@ gst_rtp_buffer_map (GstBuffer * buffer, GstMapFlags flags, GstRTPBuffer * rtp)
     goto wrong_padding;
 
   rtp->buffer = buffer;
-
-  if (n_mem == 1) {
-    /* we have mapped the buffer already, so might just as well fill in the
-     * payload pointer and size and avoid another buffer map/unmap later */
-    rtp->data[2] = rtp->map[0].data + header_len;
-    rtp->size[2] = bufsize - header_len - padding;
-  } else {
-    /* we have not yet mapped the payload */
-    rtp->data[2] = NULL;
-    rtp->size[2] = 0;
-  }
-
-  /* rtp->state = 0; *//* unused */
+  /* we have not yet mapped the payload */
+  rtp->data[2] = NULL;
+  rtp->size[2] = 0;
+  rtp->state = 0;
 
   return TRUE;
 
@@ -464,11 +443,6 @@ wrong_version:
     GST_DEBUG ("version check failed (%d != %d)", version, GST_RTP_VERSION);
     goto dump_packet;
   }
-reserved_pt:
-  {
-    GST_DEBUG ("reserved PT %d found", pt);
-    goto dump_packet;
-  }
 wrong_padding:
   {
     GST_DEBUG ("padding check failed (%" G_GSIZE_FORMAT " - %d < %d)", bufsize,
@@ -482,7 +456,7 @@ dump_packet:
     GST_MEMDUMP ("buffer", data, size);
 
     for (i = 0; i < G_N_ELEMENTS (rtp->map); ++i) {
-      if (rtp->map[i].memory != NULL)
+      if (rtp->data[i] != NULL)
         gst_buffer_unmap (buffer, &rtp->map[i]);
     }
     return FALSE;
@@ -504,12 +478,8 @@ gst_rtp_buffer_unmap (GstRTPBuffer * rtp)
   g_return_if_fail (rtp->buffer != NULL);
 
   for (i = 0; i < 4; i++) {
-    if (rtp->map[i].memory != NULL) {
+    if (rtp->data[i])
       gst_buffer_unmap (rtp->buffer, &rtp->map[i]);
-      rtp->map[i].memory = NULL;
-    }
-    rtp->data[i] = NULL;
-    rtp->size[i] = 0;
   }
   rtp->buffer = NULL;
 }
@@ -659,7 +629,7 @@ gst_rtp_buffer_pad_to (GstRTPBuffer * rtp, guint len)
  * @rtp: the RTP packet
  *
  * Check if the extension bit is set on the RTP packet in @buffer.
- *
+ * 
  * Returns: TRUE if @buffer has the extension bit set.
  */
 gboolean
@@ -682,11 +652,11 @@ gst_rtp_buffer_set_extension (GstRTPBuffer * rtp, gboolean extension)
 }
 
 /**
- * gst_rtp_buffer_get_extension_data: (skip)
+ * gst_rtp_buffer_get_extension_data:
  * @rtp: the RTP packet
- * @bits: (optional) (out): location for result bits
- * @data: (optional) (out) (array) (element-type guint8) (transfer none): location for data
- * @wordlen: (optional) (out): location for length of @data in 32 bits words
+ * @bits: (out): location for result bits
+ * @data: (out) (array) (element-type guint8) (transfer none): location for data
+ * @wordlen: (out): location for length of @data in 32 bits words
  *
  * Get the extension data. @bits will contain the extension 16 bits of custom
  * data. @data will point to the data in the extension and @wordlen will contain
@@ -694,7 +664,7 @@ gst_rtp_buffer_set_extension (GstRTPBuffer * rtp, gboolean extension)
  *
  * If @buffer did not contain an extension, this function will return %FALSE
  * with @bits, @data and @wordlen unchanged.
- *
+ * 
  * Returns: TRUE if @buffer had the extension bit set.
  */
 gboolean
@@ -719,84 +689,20 @@ gst_rtp_buffer_get_extension_data (GstRTPBuffer * rtp, guint16 * bits,
   return TRUE;
 }
 
-/**
- * gst_rtp_buffer_get_extension_bytes: (rename-to gst_rtp_buffer_get_extension_data)
- * @rtp: the RTP packet
- * @bits: (out): location for header bits
- *
- * Similar to gst_rtp_buffer_get_extension_data, but more suitable for language
- * bindings usage. @bits will contain the extension 16 bits of custom data and
- * the extension data (not including the extension header) is placed in a new
- * #GBytes structure.
- *
- * If @rtp did not contain an extension, this function will return %NULL, with
- * @bits unchanged. If there is an extension header but no extension data then
- * an empty #GBytes will be returned.
- *
- * Returns: (transfer full) (nullable): A new #GBytes if an extension header was present
- * and %NULL otherwise.
- *
- * Since: 1.2
- */
-GBytes *
-gst_rtp_buffer_get_extension_bytes (GstRTPBuffer * rtp, guint16 * bits)
-{
-  gpointer buf_data = NULL;
-  guint buf_len;
-
-  g_return_val_if_fail (rtp != NULL, FALSE);
-
-  if (!gst_rtp_buffer_get_extension_data (rtp, bits, &buf_data, &buf_len))
-    return NULL;
-
-  if (buf_len == 0) {
-    /* if no extension data is present return an empty GBytes */
-    buf_data = NULL;
-  }
-
-  /* multiply length with 4 to get length in bytes */
-  return g_bytes_new (buf_data, 4 * buf_len);
-}
-
-static gboolean
-gst_rtp_buffer_map_payload (GstRTPBuffer * rtp)
-{
-  guint hlen, plen;
-  guint idx, length;
-  gsize skip;
-
-  if (rtp->map[2].memory != NULL)
-    return TRUE;
-
-  hlen = gst_rtp_buffer_get_header_len (rtp);
-  plen = gst_buffer_get_size (rtp->buffer) - hlen - rtp->size[3];
-
-  if (!gst_buffer_find_memory (rtp->buffer, hlen, plen, &idx, &length, &skip))
-    return FALSE;
-
-  if (!gst_buffer_map_range (rtp->buffer, idx, length, &rtp->map[2],
-          rtp->map[0].flags))
-    return FALSE;
-
-  rtp->data[2] = rtp->map[2].data + skip;
-  rtp->size[2] = plen;
-
-  return TRUE;
-}
-
-/* ensure header, payload and padding are in separate memories */
+/* ensure header, payload and padding are in separate buffers */
 static void
 ensure_buffers (GstRTPBuffer * rtp)
 {
   guint i, pos;
+  gsize offset;
   gboolean changed = FALSE;
 
   /* make sure payload is mapped */
-  gst_rtp_buffer_map_payload (rtp);
+  gst_rtp_buffer_get_payload (rtp);
 
   for (i = 0, pos = 0; i < 4; i++) {
     if (rtp->size[i]) {
-      gsize offset = (guint8 *) rtp->data[i] - rtp->map[i].data;
+      offset = rtp->map[i].data - (guint8 *) rtp->data[i];
 
       if (offset != 0 || rtp->map[i].size != rtp->size[i]) {
         GstMemory *mem;
@@ -814,11 +720,9 @@ ensure_buffers (GstRTPBuffer * rtp)
   }
 
   if (changed) {
-    GstBuffer *buf = rtp->buffer;
-
     gst_rtp_buffer_unmap (rtp);
-    gst_buffer_remove_memory_range (buf, pos, -1);
-    gst_rtp_buffer_map (buf, GST_MAP_READWRITE, rtp);
+    gst_buffer_remove_memory_range (rtp->buffer, pos, -1);
+    gst_rtp_buffer_map (rtp->buffer, GST_MAP_READWRITE, rtp);
   }
 }
 
@@ -833,8 +737,6 @@ ensure_buffers (GstRTPBuffer * rtp)
  * extension header. If the existing extension data is not large enough, it will
  * be made larger.
  *
- * Will also shorten the extension data from 1.20.
- *
  * Returns: True if done.
  */
 gboolean
@@ -845,53 +747,35 @@ gst_rtp_buffer_set_extension_data (GstRTPBuffer * rtp, guint16 bits,
   guint8 *data;
   GstMemory *mem = NULL;
 
-  g_return_val_if_fail (gst_buffer_is_writable (rtp->buffer), FALSE);
-
   ensure_buffers (rtp);
 
   /* this is the size of the extension data we need */
   min_size = 4 + length * sizeof (guint32);
 
-  if (rtp->data[1] == NULL || min_size != rtp->size[1]) {
-    /* we should allocate and map the extension data */
-    if (rtp->data[1] == NULL || min_size > rtp->size[1]) {
-      GstMapInfo map;
+  /* we should allocate and map the extension data */
+  if (rtp->data[1] == NULL || min_size > rtp->size[1]) {
+    GstMapInfo map;
 
-      /* we don't have (enough) extension data, make some */
-      mem = gst_allocator_alloc (NULL, min_size, NULL);
+    /* we don't have (enough) extension data, make some */
+    mem = gst_allocator_alloc (NULL, min_size, NULL);
 
-      if (rtp->data[1]) {
-        /* copy old data & initialize the remainder of the new buffer */
-        gst_memory_map (mem, &map, GST_MAP_WRITE);
-        memcpy (map.data, rtp->data[1], rtp->size[1]);
-        if (min_size > rtp->size[1]) {
-          memset (map.data + rtp->size[1], 0, min_size - rtp->size[1]);
-        }
-        gst_memory_unmap (mem, &map);
-
-        /* unmap old */
-        gst_buffer_unmap (rtp->buffer, &rtp->map[1]);
-        gst_buffer_replace_memory (rtp->buffer, 1, mem);
-      } else {
-        /* don't leak data from uninitialized memory via the padding */
-        gst_memory_map (mem, &map, GST_MAP_WRITE);
-        memset (map.data, 0, map.size);
-        gst_memory_unmap (mem, &map);
-
-        /* we didn't have extension data, add */
-        gst_buffer_insert_memory (rtp->buffer, 1, mem);
-      }
-    } else if (min_size < rtp->size[1]) {
-      GstMemory *mem = rtp->map[1].memory;
+    if (rtp->data[1]) {
+      /* copy old data */
+      gst_memory_map (mem, &map, GST_MAP_WRITE);
+      memcpy (map.data, rtp->data[1], rtp->size[1]);
+      gst_memory_unmap (mem, &map);
 
+      /* unmap old */
       gst_buffer_unmap (rtp->buffer, &rtp->map[1]);
-      gst_memory_resize (mem, 0, min_size);
+      gst_buffer_replace_memory (rtp->buffer, 1, mem);
+    } else {
+      /* we didn't have extension data, add */
+      gst_buffer_insert_memory (rtp->buffer, 1, mem);
     }
 
-    /* map extension data again */
-    if (!gst_buffer_map_range (rtp->buffer, 1, 1, &rtp->map[1],
-            GST_MAP_READWRITE))
-      g_assert_not_reached ();  /* checked in gst_rtp_buffer_map() / ensure_buffers already */
+    /* map new */
+    gst_memory_map (mem, &rtp->map[1], GST_MAP_READWRITE);
+    gst_memory_ref (mem);
     rtp->data[1] = rtp->map[1].data;
     rtp->size[1] = rtp->map[1].size;
   }
@@ -907,43 +791,12 @@ gst_rtp_buffer_set_extension_data (GstRTPBuffer * rtp, guint16 bits,
   return TRUE;
 }
 
-/**
- * gst_rtp_buffer_remove_extension_data:
- * @rtp: the RTP packet
- *
- * Unsets the extension bit of the RTP buffer and removes the extension header
- * and data.
- *
- * If the RTP buffer has no header extension data, the action has no effect.
- * The RTP buffer must be mapped READWRITE only once and the underlying
- * GstBuffer must be writable.
- *
- * Since: 1.20
- */
-void
-gst_rtp_buffer_remove_extension_data (GstRTPBuffer * rtp)
-{
-  g_return_if_fail (gst_buffer_is_writable (rtp->buffer));
-  g_return_if_fail (rtp->map[0].flags & GST_MAP_WRITE);
-
-  if (rtp->data[1] != NULL) {
-    GstBuffer *buf = rtp->buffer;
-
-    ensure_buffers (rtp);
-
-    GST_RTP_HEADER_EXTENSION (rtp->data[0]) = FALSE;
-    gst_rtp_buffer_unmap (rtp);
-    gst_buffer_remove_memory (buf, 1);
-    gst_rtp_buffer_map (buf, GST_MAP_READWRITE, rtp);
-  }
-}
-
 /**
  * gst_rtp_buffer_get_ssrc:
  * @rtp: the RTP packet
  *
  * Get the SSRC of the RTP packet in @buffer.
- *
+ * 
  * Returns: the SSRC of @buffer in host order.
  */
 guint32
@@ -970,7 +823,7 @@ gst_rtp_buffer_set_ssrc (GstRTPBuffer * rtp, guint32 ssrc)
  * @rtp: the RTP packet
  *
  * Get the CSRC count of the RTP packet in @buffer.
- *
+ * 
  * Returns: the CSRC count of @buffer.
  */
 guint8
@@ -985,7 +838,7 @@ gst_rtp_buffer_get_csrc_count (GstRTPBuffer * rtp)
  * @idx: the index of the CSRC to get
  *
  * Get the CSRC at index @idx in @buffer.
- *
+ * 
  * Returns: the CSRC at index @idx in host order.
  */
 guint32
@@ -1139,9 +992,9 @@ gst_rtp_buffer_set_timestamp (GstRTPBuffer * rtp, guint32 timestamp)
  *
  * Create a subbuffer of the payload of the RTP packet in @buffer. @offset bytes
  * are skipped in the payload and the subbuffer will be of size @len.
- * If @len is -1 the total payload starting from @offset is subbuffered.
+ * If @len is -1 the total payload starting from @offset if subbuffered.
  *
- * Returns: (transfer full): A new buffer with the specified data of the payload.
+ * Returns: A new buffer with the specified data of the payload.
  */
 GstBuffer *
 gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
@@ -1151,7 +1004,7 @@ gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
 
   plen = gst_rtp_buffer_get_payload_len (rtp);
   /* we can't go past the length */
-  if (G_UNLIKELY (offset > plen))
+  if (G_UNLIKELY (offset >= plen))
     goto wrong_offset;
 
   /* apply offset */
@@ -1168,7 +1021,7 @@ gst_rtp_buffer_get_payload_subbuffer (GstRTPBuffer * rtp, guint offset,
   /* ERRORS */
 wrong_offset:
   {
-    g_warning ("offset=%u should be less than plen=%u", offset, plen);
+    g_warning ("offset=%u should be less then plen=%u", offset, plen);
     return NULL;
   }
 }
@@ -1181,7 +1034,7 @@ wrong_offset:
  * will internally create a subbuffer of @buffer so that a memcpy can be
  * avoided.
  *
- * Returns: (transfer full): A new buffer with the data of the payload.
+ * Returns: A new buffer with the data of the payload.
  */
 GstBuffer *
 gst_rtp_buffer_get_payload_buffer (GstRTPBuffer * rtp)
@@ -1205,53 +1058,41 @@ gst_rtp_buffer_get_payload_len (GstRTPBuffer * rtp)
 }
 
 /**
- * gst_rtp_buffer_get_payload: (skip)
+ * gst_rtp_buffer_get_payload:
  * @rtp: the RTP packet
  *
  * Get a pointer to the payload data in @buffer. This pointer is valid as long
  * as a reference to @buffer is held.
  *
- * Returns: (array) (element-type guint8) (transfer none) (nullable): A pointer
+ * Returns: (array) (element-type guint8) (transfer none): A pointer
  * to the payload data in @buffer.
  */
 gpointer
 gst_rtp_buffer_get_payload (GstRTPBuffer * rtp)
 {
+  guint hlen, plen;
+  guint idx, length;
+  gsize skip;
+
   if (rtp->data[2])
     return rtp->data[2];
 
-  if (!gst_rtp_buffer_map_payload (rtp))
+  hlen = gst_rtp_buffer_get_header_len (rtp);
+  plen = gst_buffer_get_size (rtp->buffer) - hlen - rtp->size[3];
+
+  if (!gst_buffer_find_memory (rtp->buffer, hlen, plen, &idx, &length, &skip))
     return NULL;
 
+  if (!gst_buffer_map_range (rtp->buffer, idx, length, &rtp->map[2],
+          rtp->map[0].flags))
+    return NULL;
+
+  rtp->data[2] = rtp->map[2].data + skip;
+  rtp->size[2] = plen;
+
   return rtp->data[2];
 }
 
-/**
- * gst_rtp_buffer_get_payload_bytes: (rename-to gst_rtp_buffer_get_payload)
- * @rtp: the RTP packet
- *
- * Similar to gst_rtp_buffer_get_payload, but more suitable for language
- * bindings usage. The return value is a pointer to a #GBytes structure
- * containing the payload data in @rtp.
- *
- * Returns: (transfer full) (nullable): A new #GBytes containing the payload data in @rtp.
- *
- * Since: 1.2
- */
-GBytes *
-gst_rtp_buffer_get_payload_bytes (GstRTPBuffer * rtp)
-{
-  gpointer data;
-
-  g_return_val_if_fail (rtp != NULL, NULL);
-
-  data = gst_rtp_buffer_get_payload (rtp);
-  if (data == NULL)
-    return NULL;
-
-  return g_bytes_new (data, gst_rtp_buffer_get_payload_len (rtp));
-}
-
 /**
  * gst_rtp_buffer_default_clock_rate:
  * @payload_type: the static payload type
@@ -1293,95 +1134,92 @@ gst_rtp_buffer_default_clock_rate (guint8 payload_type)
 gint
 gst_rtp_buffer_compare_seqnum (guint16 seqnum1, guint16 seqnum2)
 {
-  /* See http://en.wikipedia.org/wiki/Serial_number_arithmetic
-   * for an explanation why this does the right thing even for
-   * wraparounds, under the assumption that the difference is
-   * never bigger than 2**15 sequence numbers
-   */
   return (gint16) (seqnum2 - seqnum1);
 }
 
 /**
  * gst_rtp_buffer_ext_timestamp:
- * @exttimestamp: (inout): a previous extended timestamp
+ * @exttimestamp: a previous extended timestamp
  * @timestamp: a new timestamp
  *
- * Update the @exttimestamp field with the extended timestamp of @timestamp
- * For the first call of the method, @exttimestamp should point to a location
- * with a value of -1.
+ * Update the @exttimestamp field with @timestamp. For the first call of the
+ * method, @exttimestamp should point to a location with a value of -1.
  *
- * This function is able to handle both forward and backward timestamps taking
- * into account:
- *   - timestamp wraparound making sure that the returned value is properly increased.
- *   - timestamp unwraparound making sure that the returned value is properly decreased.
+ * This function makes sure that the returned value is a constantly increasing
+ * value even in the case where there is a timestamp wraparound.
  *
- * Returns: The extended timestamp of @timestamp or 0 if the result can't go anywhere backwards.
+ * Returns: The extended timestamp of @timestamp.
  */
 guint64
 gst_rtp_buffer_ext_timestamp (guint64 * exttimestamp, guint32 timestamp)
 {
-  guint64 result, ext;
+  guint64 result, diff, ext;
 
   g_return_val_if_fail (exttimestamp != NULL, -1);
 
   ext = *exttimestamp;
 
   if (ext == -1) {
-    result = (G_GUINT64_CONSTANT (1) << 32) + timestamp;
+    result = timestamp;
   } else {
     /* pick wraparound counter from previous timestamp and add to new timestamp */
-    result = timestamp + (ext & ~(G_GUINT64_CONSTANT (0xffffffff)));
+    result = timestamp + (ext & ~(G_GINT64_CONSTANT (0xffffffff)));
 
     /* check for timestamp wraparound */
-    if (result < ext) {
-      guint64 diff = ext - result;
+    if (result < ext)
+      diff = ext - result;
+    else
+      diff = result - ext;
 
-      if (diff > G_MAXINT32) {
-        /* timestamp went backwards more than allowed, we wrap around and get
-         * updated extended timestamp. */
-        result += (G_GUINT64_CONSTANT (1) << 32);
-      }
-    } else {
-      guint64 diff = result - ext;
-
-      if (diff > G_MAXINT32) {
-        if (result < (G_GUINT64_CONSTANT (1) << 32)) {
-          GST_WARNING
-              ("Cannot unwrap, any wrapping took place yet. Returning 0 without updating extended timestamp.");
-          return 0;
-        } else {
-          /* timestamp went forwards more than allowed, we unwrap around and get
-           * updated extended timestamp. */
-          result -= (G_GUINT64_CONSTANT (1) << 32);
-          /* We don't want the extended timestamp storage to go back, ever */
-          return result;
-        }
-      }
+    if (diff > G_MAXINT32) {
+      /* timestamp went backwards more than allowed, we wrap around and get
+       * updated extended timestamp. */
+      result += (G_GINT64_CONSTANT (1) << 32);
     }
   }
-
   *exttimestamp = result;
 
   return result;
 }
 
+/**
+ * gst_rtp_buffer_get_extension_onebyte_header:
+ * @rtp: the RTP packet
+ * @id: The ID of the header extension to be read (between 1 and 14).
+ * @nth: Read the nth extension packet with the requested ID
+ * @data: (out) (array length=size) (element-type guint8) (transfer none):
+ *   location for data
+ * @size: (out): the size of the data in bytes
+ *
+ * Parses RFC 5285 style header extensions with a one byte header. It will
+ * return the nth extension with the requested id.
+ *
+ * Returns: TRUE if @buffer had the requested header extension
+ */
 
-static gboolean
-_get_extension_onebyte_header (const guint8 * pdata, guint len,
-    guint16 bit_pattern, guint8 id, guint nth, gpointer * data, guint * size)
+gboolean
+gst_rtp_buffer_get_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
+    guint nth, gpointer * data, guint * size)
 {
+  guint16 bits;
+  guint8 *pdata;
+  guint wordlen;
   gulong offset = 0;
   guint count = 0;
 
   g_return_val_if_fail (id > 0 && id < 15, FALSE);
 
-  if (bit_pattern != 0xBEDE)
+  if (!gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata,
+          &wordlen))
+    return FALSE;
+
+  if (bits != 0xBEDE)
     return FALSE;
 
   for (;;) {
     guint8 read_id, read_len;
 
-    if (offset + 1 >= len)
+    if (offset + 1 >= wordlen * 4)
       break;
 
     read_id = GST_READ_UINT8 (pdata + offset) >> 4;
@@ -1397,14 +1235,14 @@ _get_extension_onebyte_header (const guint8 * pdata, guint len,
       break;
 
     /* Ignore extension headers where the size does not fit */
-    if (offset + read_len > len)
+    if (offset + read_len > wordlen * 4)
       break;
 
     /* If we have the right one */
     if (id == read_id) {
       if (nth == count) {
         if (data)
-          *data = (gpointer) & pdata[offset];
+          *data = pdata + offset;
         if (size)
           *size = read_len;
 
@@ -1415,86 +1253,23 @@ _get_extension_onebyte_header (const guint8 * pdata, guint len,
     }
     offset += read_len;
 
-    if (offset >= len)
+    if (offset >= wordlen * 4)
       break;
   }
 
   return FALSE;
 }
 
-
 /**
- * gst_rtp_buffer_get_extension_onebyte_header_from_bytes:
- * @bytes: #GBytes
- * @bit_pattern: The bit-pattern. Anything but 0xBEDE is rejected.
+ * gst_rtp_buffer_get_extension_twobytes_header:
+ * @rtp: the RTP packet
+ * @appbits: (out): Application specific bits
  * @id: The ID of the header extension to be read (between 1 and 14).
  * @nth: Read the nth extension packet with the requested ID
  * @data: (out) (array length=size) (element-type guint8) (transfer none):
  *   location for data
  * @size: (out): the size of the data in bytes
  *
- * Similar to gst_rtp_buffer_get_extension_onebyte_header, but working
- * on the #GBytes you get from gst_rtp_buffer_get_extension_bytes.
- * Parses RFC 5285 style header extensions with a one byte header. It will
- * return the nth extension with the requested id.
- *
- * Returns: TRUE if @bytes had the requested header extension
- *
- * Since: 1.18
- */
-gboolean
-gst_rtp_buffer_get_extension_onebyte_header_from_bytes (GBytes * bytes,
-    guint16 bit_pattern, guint8 id, guint nth, gpointer * data, guint * size)
-{
-  const guint8 *pdata = g_bytes_get_data (bytes, NULL);
-  gsize len = g_bytes_get_size (bytes);
-  return _get_extension_onebyte_header (pdata, len, bit_pattern, id, nth, data,
-      size);
-}
-
-
-/**
- * gst_rtp_buffer_get_extension_onebyte_header:
- * @rtp: the RTP packet
- * @id: The ID of the header extension to be read (between 1 and 14).
- * @nth: Read the nth extension packet with the requested ID
- * @data: (optional) (out) (array length=size) (element-type guint8) (transfer none):
- *   location for data
- * @size: (optional) (out): the size of the data in bytes
- *
- * Parses RFC 5285 style header extensions with a one byte header. It will
- * return the nth extension with the requested id.
- *
- * Returns: TRUE if @buffer had the requested header extension
- */
-
-gboolean
-gst_rtp_buffer_get_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
-    guint nth, gpointer * data, guint * size)
-{
-  guint16 bit_pattern;
-  guint8 *pdata;
-  guint wordlen;
-
-  if (!gst_rtp_buffer_get_extension_data (rtp, &bit_pattern, (gpointer) & pdata,
-          &wordlen))
-    return FALSE;
-
-  return _get_extension_onebyte_header (pdata, wordlen * 4, bit_pattern, id,
-      nth, data, size);
-}
-
-
-/**
- * gst_rtp_buffer_get_extension_twobytes_header:
- * @rtp: the RTP packet
- * @appbits: (optional) (out): Application specific bits
- * @id: The ID of the header extension to be read (between 1 and 14).
- * @nth: Read the nth extension packet with the requested ID
- * @data: (optional) (out) (array length=size) (element-type guint8) (transfer none):
- *   location for data
- * @size: (optional) (out): the size of the data in bytes
- *
  * Parses RFC 5285 style header extensions with a two bytes header. It will
  * return the nth extension with the requested id.
  *
@@ -1561,20 +1336,19 @@ gst_rtp_buffer_get_extension_twobytes_header (GstRTPBuffer * rtp,
   return FALSE;
 }
 
-static gboolean
-get_onebyte_header_end_offset (guint8 * pdata, guint wordlen, guint * offset)
+static guint
+get_onebyte_header_end_offset (guint8 * pdata, guint wordlen)
 {
+  guint offset = 0;
   guint bytelen = wordlen * 4;
   guint paddingcount = 0;
 
-  *offset = 0;
-
-  while (*offset + 1 < bytelen) {
+  while (offset + 1 < bytelen) {
     guint8 read_id, read_len;
 
-    read_id = GST_READ_UINT8 (pdata + *offset) >> 4;
-    read_len = (GST_READ_UINT8 (pdata + *offset) & 0x0F) + 1;
-    *offset += 1;
+    read_id = GST_READ_UINT8 (pdata + offset) >> 4;
+    read_len = (GST_READ_UINT8 (pdata + offset) & 0x0F) + 1;
+    offset += 1;
 
     /* ID 0 means its padding, skip */
     if (read_id == 0) {
@@ -1586,21 +1360,17 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen, guint * offset)
 
     /* ID 15 is special and means we should stop parsing */
     /* It also means we can't add an extra packet */
-    if (read_id == 15) {
-      return FALSE;
-    }
+    if (read_id == 15)
+      return 0;
 
     /* Ignore extension headers where the size does not fit */
-    if (*offset + read_len > bytelen) {
-      return FALSE;
-    }
+    if (offset + read_len > bytelen)
+      return 0;
 
-    *offset += read_len;
+    offset += read_len;
   }
 
-  *offset -= paddingcount;
-
-  return TRUE;
+  return offset - paddingcount;
 }
 
 /**
@@ -1614,7 +1384,7 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen, guint * offset)
  * RTP header. If there is already a RFC 5285 header extension with a one byte
  * header, the new extension will be appended.
  * It will not work if there is already a header extension that does not follow
- * the mechanism described in RFC 5285 or if there is a header extension with
+ * the mecanism described in RFC 5285 or if there is a header extension with
  * a two bytes header as described in RFC 5285. In that case, use
  * gst_rtp_buffer_add_extension_twobytes_header()
  *
@@ -1623,12 +1393,11 @@ get_onebyte_header_end_offset (guint8 * pdata, guint wordlen, guint * offset)
 
 gboolean
 gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
-    gconstpointer data, guint size)
+    gpointer data, guint size)
 {
   guint16 bits;
   guint8 *pdata = 0;
-  guint wordlen = 0;
-  guint wordlen_new;
+  guint wordlen;
   gboolean has_bit;
   guint extlen, offset = 0;
 
@@ -1643,17 +1412,17 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
     if (bits != 0xBEDE)
       return FALSE;
 
-    if (!get_onebyte_header_end_offset (pdata, wordlen, &offset))
+    offset = get_onebyte_header_end_offset (pdata, wordlen);
+    if (offset == 0)
       return FALSE;
   }
 
   /* the required size of the new extension data */
   extlen = offset + size + 1;
   /* calculate amount of words */
-  wordlen_new = extlen / 4 + ((extlen % 4) ? 1 : 0);
-  wordlen_new = MAX (wordlen_new, wordlen);
+  wordlen = extlen / 4 + ((extlen % 4) ? 1 : 0);
 
-  gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen_new);
+  gst_rtp_buffer_set_extension_data (rtp, 0xBEDE, wordlen);
   gst_rtp_buffer_get_extension_data (rtp, &bits, (gpointer) & pdata, &wordlen);
 
   pdata += offset;
@@ -1669,7 +1438,7 @@ gst_rtp_buffer_add_extension_onebyte_header (GstRTPBuffer * rtp, guint8 id,
 
 
 static guint
-get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
+get_twobytes_header_end_offset (guint8 * pdata, guint wordlen)
 {
   guint offset = 0;
   guint bytelen = wordlen * 4;
@@ -1714,7 +1483,7 @@ get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
  * RTP header. If there is already a RFC 5285 header extension with a two bytes
  * header, the new extension will be appended.
  * It will not work if there is already a header extension that does not follow
- * the mechanism described in RFC 5285 or if there is a header extension with
+ * the mecanism described in RFC 5285 or if there is a header extension with
  * a one byte header as described in RFC 5285. In that case, use
  * gst_rtp_buffer_add_extension_onebyte_header()
  *
@@ -1723,7 +1492,7 @@ get_twobytes_header_end_offset (const guint8 * pdata, guint wordlen)
 
 gboolean
 gst_rtp_buffer_add_extension_twobytes_header (GstRTPBuffer * rtp,
-    guint8 appbits, guint8 id, gconstpointer data, guint size)
+    guint8 appbits, guint8 id, gpointer data, guint size)
 {
   guint16 bits;
   guint8 *pdata = 0;
