@@ -16,13 +16,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:gsttagxmp
- * @title: GstXmptag
  * @short_description: tag mappings and support functions for plugins
  *                     dealing with xmp packets
  * @see_also: #GstTagList
@@ -45,24 +44,6 @@
 #include <time.h>
 #include <ctype.h>
 
-#define GST_CAT_DEFAULT gst_tag_ensure_debug_category()
-
-static GstDebugCategory *
-gst_tag_ensure_debug_category (void)
-{
-  static gsize cat_gonce = 0;
-
-  if (g_once_init_enter (&cat_gonce)) {
-    GstDebugCategory *cat = NULL;
-
-    GST_DEBUG_CATEGORY_INIT (cat, "xmp-tags", 0, "XMP GstTag helper functions");
-
-    g_once_init_leave (&cat_gonce, (gsize) cat);
-  }
-
-  return (GstDebugCategory *) cat_gonce;
-}
-
 static const gchar *schema_list[] = {
   "dc",
   "xap",
@@ -79,7 +60,7 @@ static const gchar *schema_list[] = {
  *
  * Gets the list of supported schemas in the xmp lib
  *
- * Returns: (transfer none) (array zero-terminated=1): a %NULL terminated array of strings with the
+ * Returns: (transfer none): a %NULL terminated array of strings with the
  *     schema names
  */
 const gchar **
@@ -213,7 +194,7 @@ typedef GHashTable GstXmpSchema;
 #define gst_xmp_schema_lookup g_hash_table_lookup
 #define gst_xmp_schema_insert g_hash_table_insert
 static GstXmpSchema *
-gst_xmp_schema_new (void)
+gst_xmp_schema_new ()
 {
   return g_hash_table_new (g_direct_hash, g_direct_equal);
 }
@@ -276,7 +257,7 @@ gst_xmp_tag_create (const gchar * gst_tag, const gchar * xmp_tag,
 {
   XmpTag *xmpinfo;
 
-  xmpinfo = g_new (XmpTag, 1);
+  xmpinfo = g_slice_new (XmpTag);
   xmpinfo->gst_tag = gst_tag;
   xmpinfo->tag_name = xmp_tag;
   xmpinfo->type = xmp_type;
@@ -485,8 +466,7 @@ deserialize_exif_gps_coordinate (XmpTag * xmptag, GstTagList * taglist,
 
   /* check if it uses ,SS or .mm */
   if (strchr (current, ',') != NULL) {
-    if (!sscanf (current, "%d,%d%c", &m, &s, &c))
-      goto error;
+    sscanf (current, "%d,%d%c", &m, &s, &c);
   } else {
     gchar *copy = g_strdup (current);
     gint len = strlen (copy);
@@ -653,7 +633,7 @@ deserialize_exif_altitude (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_free (ptag);
+  g_slice_free (PendingXmpTag, ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -748,7 +728,7 @@ deserialize_exif_gps_speed (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_free (ptag);
+  g_slice_free (PendingXmpTag, ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -811,11 +791,11 @@ deserialize_exif_gps_direction (XmpTag * xmptag, GstTagList * taglist,
   }
 
   if (!dir_str) {
-    GST_WARNING ("Missing GPSImgDirection tag");
+    GST_WARNING ("Missing %s tag", dir_str);
     return;
   }
   if (!dirref_str) {
-    GST_WARNING ("Missing GPSImgDirectionRef tag");
+    GST_WARNING ("Missing %s tag", dirref_str);
     return;
   }
 
@@ -842,7 +822,7 @@ deserialize_exif_gps_direction (XmpTag * xmptag, GstTagList * taglist,
 
   /* clean up entry */
   g_free (ptag->str);
-  g_free (ptag);
+  g_slice_free (PendingXmpTag, ptag);
   *pending_tags = g_slist_delete_link (*pending_tags, entry);
 }
 
@@ -932,7 +912,7 @@ deserialize_tiff_orientation (XmpTag * xmptag, GstTagList * taglist,
 }
 
 
-/* look at this page for additional schemas
+/* look at this page for addtional schemas
  * http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/XMP.html
  */
 static gpointer
@@ -941,8 +921,6 @@ _init_xmp_tag_map (gpointer user_data)
   XmpTag *xmpinfo;
   GstXmpSchema *schema;
 
-  gst_tag_register_musicbrainz_tags ();
-
   __xmp_schemas = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   /* add the maps */
@@ -1066,7 +1044,7 @@ _init_xmp_tag_map (gpointer user_data)
 }
 
 static void
-xmp_tags_initialize (void)
+xmp_tags_initialize ()
 {
   static GOnce my_once = G_ONCE_INIT;
   g_once (&my_once, (GThreadFunc) _init_xmp_tag_map, NULL);
@@ -1080,7 +1058,7 @@ struct _GstXmpNamespaceMatch
 
   /*
    * Stores extra namespaces for array tags
-   * The namespaces should be written in the form:
+   * The namespaces should be writen in the form:
    *
    * xmlns:XpTo="http://some.org/your/ns/name/ (next ones)"
    */
@@ -1175,7 +1153,7 @@ read_one_tag (GstTagList * list, XmpTag * xmptag,
  *
  * Parse a xmp packet into a taglist.
  *
- * Returns: (transfer full) (nullable): new taglist or %NULL, free the list when done
+ * Returns: new taglist or %NULL, free the list when done
  */
 GstTagList *
 gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
@@ -1185,7 +1163,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
   gchar *xps, *xp1, *xp2, *xpe, *ns, *ne;
   gsize len, max_ft_len;
   gboolean in_tag;
-  gchar *part = NULL, *pp;
+  gchar *part, *pp;
   guint i;
   XmpTag *last_xmp_tag = NULL;
   GSList *pending_tags = NULL;
@@ -1241,6 +1219,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
   if (len < max_ft_len)
     goto missing_footer;
 
+  GST_DEBUG ("checking footer: [%s]", &xps[len - max_ft_len]);
   xp2 = g_strstr_len (&xps[len - max_ft_len], max_ft_len, "<?xpacket ");
   if (!xp2)
     goto missing_footer;
@@ -1252,7 +1231,7 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
   while (*xp1 != '<' && xp1 < xpe)
     xp1++;
 
-  /* no tag can be longer than the whole buffer */
+  /* no tag can be longer that the whole buffer */
   part = g_malloc (xp2 - xp1);
   list = gst_tag_list_new_empty ();
 
@@ -1317,7 +1296,6 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
                 }
                 if (ns_match[i].ns_prefix) {
                   if (strcmp (ns_map[i].original_ns, &as[6])) {
-                    g_free (ns_map[i].gstreamer_ns);
                     ns_map[i].gstreamer_ns = g_strdup (&as[6]);
                   }
                 }
@@ -1356,11 +1334,11 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
                   /* we shouldn't find a xmp structure here */
                   g_assert (xmp_tag->gst_tag != NULL);
 
-                  ptag = g_new (PendingXmpTag, 1);
+                  ptag = g_slice_new (PendingXmpTag);
                   ptag->xmp_tag = xmp_tag;
                   ptag->str = g_strdup (v);
 
-                  pending_tags = g_slist_prepend (pending_tags, ptag);
+                  pending_tags = g_slist_append (pending_tags, ptag);
                 }
               }
               /* restore chars overwritten by '\0' */
@@ -1449,11 +1427,11 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
 
             context_tag = last_xmp_tag;
           } else {
-            ptag = g_new (PendingXmpTag, 1);
+            ptag = g_slice_new (PendingXmpTag);
             ptag->xmp_tag = last_xmp_tag;
             ptag->str = g_strdup (part);
 
-            pending_tags = g_slist_prepend (pending_tags, ptag);
+            pending_tags = g_slist_append (pending_tags, ptag);
           }
         }
       }
@@ -1464,10 +1442,6 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
     }
   }
 
-  pending_tags = g_slist_reverse (pending_tags);
-
-  GST_DEBUG ("Done accumulating tags, now handling them");
-
   while (pending_tags) {
     PendingXmpTag *ptag = (PendingXmpTag *) pending_tags->data;
 
@@ -1476,20 +1450,17 @@ gst_tag_list_from_xmp_buffer (GstBuffer * buffer)
     read_one_tag (list, ptag->xmp_tag, ptag->str, &pending_tags);
 
     g_free (ptag->str);
-    g_free (ptag);
+    g_slice_free (PendingXmpTag, ptag);
   }
 
   GST_INFO ("xmp packet parsed, %d entries", gst_tag_list_n_tags (list));
 
-out:
-
   /* free resources */
   i = 0;
   while (ns_map[i].original_ns) {
     g_free (ns_map[i].gstreamer_ns);
     i++;
   }
-
   g_free (part);
 
   gst_buffer_unmap (buffer, &info);
@@ -1499,15 +1470,13 @@ out:
   /* Errors */
 missing_header:
   GST_WARNING ("malformed xmp packet header");
-  goto out;
+  return NULL;
 missing_footer:
   GST_WARNING ("malformed xmp packet footer");
-  goto out;
+  return NULL;
 broken_xml:
   GST_WARNING ("malformed xml tag: %s", part);
-  gst_tag_list_unref (list);
-  list = NULL;
-  goto out;
+  return NULL;
 }
 
 
@@ -1700,14 +1669,13 @@ write_one_tag (const GstTagList * list, XmpTag * xmp_tag, gpointer user_data)
  * gst_tag_list_to_xmp_buffer:
  * @list: tags
  * @read_only: does the container forbid inplace editing
- * @schemas: (array zero-terminated=1):
- *     %NULL terminated array of schemas to be used on serialization
+ * @schemas: %NULL terminated array of schemas to be used on serialization
  *
  * Formats a taglist as a xmp packet using only the selected
  * schemas. An empty list (%NULL) means that all schemas should
  * be used
  *
- * Returns: (transfer full) (nullable): new buffer or %NULL, unref the buffer when done
+ * Returns: new buffer or %NULL, unref the buffer when done
  */
 GstBuffer *
 gst_tag_list_to_xmp_buffer (const GstTagList * list, gboolean read_only,
