@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /* Super-primitive SSA parser - we just want the text and ignore
@@ -24,17 +24,12 @@
 #include "config.h"
 #endif
 
-#include <ctype.h>              /* isspace() */
-#include <stdio.h>              /* sscanf() */
 #include <stdlib.h>             /* atoi() */
 #include <string.h>
 
 #include "gstssaparse.h"
-#include "gstsubparseelements.h"
-
 
 GST_DEBUG_CATEGORY_STATIC (ssa_parse_debug);
-#undef GST_CAT_DEFAULT
 #define GST_CAT_DEFAULT ssa_parse_debug
 
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -51,9 +46,6 @@ static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
 
 #define gst_ssa_parse_parent_class parent_class
 G_DEFINE_TYPE (GstSsaParse, gst_ssa_parse, GST_TYPE_ELEMENT);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (ssaparse, "ssaparse",
-    GST_RANK_PRIMARY, GST_TYPE_SSA_PARSE, sub_parse_element_init (plugin));
-
 
 static GstStateChangeReturn gst_ssa_parse_change_state (GstElement *
     element, GstStateChange transition);
@@ -91,6 +83,8 @@ gst_ssa_parse_init (GstSsaParse * parse)
       GST_DEBUG_FUNCPTR (gst_ssa_parse_src_event));
   gst_element_add_pad (GST_ELEMENT (parse), parse->srcpad);
   gst_pad_use_fixed_caps (parse->srcpad);
+  gst_pad_set_caps (parse->srcpad,
+      gst_static_pad_template_get_caps (&src_templ));
 
   parse->ini = NULL;
   parse->framed = FALSE;
@@ -105,10 +99,12 @@ gst_ssa_parse_class_init (GstSsaParseClass * klass)
 
   object_class->dispose = gst_ssa_parse_dispose;
 
-  gst_element_class_add_static_pad_template (element_class, &sink_templ);
-  gst_element_class_add_static_pad_template (element_class, &src_templ);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_templ));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_templ));
   gst_element_class_set_static_metadata (element_class,
-      "SSA Subtitle Parser", "Codec/Decoder/Subtitle",
+      "SSA Subtitle Parser", "Codec/Parser/Subtitle",
       "Parses SSA subtitle streams",
       "Tim-Philipp Müller <tim centricular net>");
 
@@ -154,11 +150,10 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
   const GValue *val;
   GstStructure *s;
   const guchar bom_utf8[] = { 0xEF, 0xBB, 0xBF };
-  const gchar *end;
   GstBuffer *priv;
   GstMapInfo map;
   gchar *ptr;
-  gsize left, bad_offset;
+  gsize left;
   gboolean ret;
 
   s = gst_caps_get_structure (caps, 0);
@@ -177,12 +172,7 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
 
   gst_buffer_ref (priv);
 
-  if (!gst_buffer_map (priv, &map, GST_MAP_READ)) {
-    gst_buffer_unref (priv);
-    return FALSE;
-  }
-
-  GST_MEMDUMP_OBJECT (parse, "init section", map.data, map.size);
+  gst_buffer_map (priv, &map, GST_MAP_READ);
 
   ptr = (gchar *) map.data;
   left = map.size;
@@ -196,17 +186,10 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
   if (!strstr (ptr, "[Script Info]"))
     goto invalid_init;
 
-  if (!g_utf8_validate (ptr, left, &end)) {
-    bad_offset = (gsize) (end - ptr);
-    GST_WARNING_OBJECT (parse, "Init section is not valid UTF-8. Problem at "
-        "byte offset %" G_GSIZE_FORMAT, bad_offset);
-    /* continue with valid UTF-8 data */
-    left = bad_offset;
-  }
+  if (!g_utf8_validate (ptr, left, NULL))
+    goto invalid_utf8;
 
   /* FIXME: parse initial section */
-  if (parse->ini)
-    g_free (parse->ini);
   parse->ini = g_strndup (ptr, left);
   GST_LOG_OBJECT (parse, "Init section:\n%s", parse->ini);
 
@@ -229,6 +212,13 @@ invalid_init:
     gst_buffer_unref (priv);
     return FALSE;
   }
+invalid_utf8:
+  {
+    GST_WARNING_OBJECT (parse, "Init section is not valid UTF-8");
+    gst_buffer_unmap (priv, &map);
+    gst_buffer_unref (priv);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -244,7 +234,7 @@ gst_ssa_parse_remove_override_codes (GstSsaParse * parse, gchar * txt)
       return removed_any;
     }
     /* move terminating NUL character forward as well */
-    memmove (t, end + 1, strlen (end + 1) + 1);
+    g_memmove (t, end + 1, strlen (end + 1) + 1);
     removed_any = TRUE;
   }
 
