@@ -14,24 +14,23 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-tcpserversink
- * @title: tcpserversink
  * @see_also: #multifdsink
  *
- * ## Example launch line (server):
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 fdsrc fd=1 ! tcpserversink port=3000
- * ]|
- * ## Example launch line (client):
- * |[
- * gst-launch-1.0 tcpclientsrc port=3000 ! fdsink fd=2
- * ]|
- *
+ * # server:
+ * gst-launch fdsrc fd=1 ! tcpserversink port=3000
+ * # client:
+ * gst-launch tcpclientsrc port=3000 ! fdsink fd=2
+ * ]| 
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,8 +39,9 @@
 #include <gst/gst-i18n-plugin.h>
 #include <string.h>             /* memset */
 
-#include "gsttcpelements.h"
+#include "gsttcp.h"
 #include "gsttcpserversink.h"
+#include "gsttcp-marshal.h"
 
 #define TCP_BACKLOG             5
 
@@ -71,8 +71,6 @@ static void gst_tcp_server_sink_get_property (GObject * object, guint prop_id,
 #define gst_tcp_server_sink_parent_class parent_class
 G_DEFINE_TYPE (GstTCPServerSink, gst_tcp_server_sink,
     GST_TYPE_MULTI_SOCKET_SINK);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (tcpserversink, "tcpserversink",
-    GST_RANK_NONE, GST_TYPE_TCP_SERVER_SINK, tcp_element_init (plugin));
 
 static void
 gst_tcp_server_sink_class_init (GstTCPServerSinkClass * klass)
@@ -89,8 +87,6 @@ gst_tcp_server_sink_class_init (GstTCPServerSinkClass * klass)
   gobject_class->get_property = gst_tcp_server_sink_get_property;
   gobject_class->finalize = gst_tcp_server_sink_finalize;
 
-  /* FIXME 2.0: Rename this to bind-address, host does not make much
-   * sense here */
   g_object_class_install_property (gobject_class, PROP_HOST,
       g_param_spec_string ("host", "host", "The host/IP to listen on",
           TCP_DEFAULT_HOST, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
@@ -155,7 +151,6 @@ gst_tcp_server_sink_finalize (GObject * gobject)
 static gboolean
 gst_tcp_server_sink_handle_server_read (GstTCPServerSink * sink)
 {
-  GstMultiSinkHandle handle;
   GSocket *client_socket;
   GError *err = NULL;
 
@@ -165,33 +160,24 @@ gst_tcp_server_sink_handle_server_read (GstTCPServerSink * sink)
   if (!client_socket)
     goto accept_failed;
 
-  handle.socket = client_socket;
-  /* gst_multi_handle_sink_add does not take ownership of client_socket */
-  gst_multi_handle_sink_add (GST_MULTI_HANDLE_SINK (sink), handle);
+  gst_multi_handle_sink_add (GST_MULTI_HANDLE_SINK (sink),
+      (GstMultiSinkHandle) client_socket);
 
 #ifndef GST_DISABLE_GST_DEBUG
   {
     GInetSocketAddress *addr =
         G_INET_SOCKET_ADDRESS (g_socket_get_remote_address (client_socket,
             NULL));
-    if (addr) {
-      gchar *ip =
-          g_inet_address_to_string (g_inet_socket_address_get_address (addr));
+    gchar *ip =
+        g_inet_address_to_string (g_inet_socket_address_get_address (addr));
 
-      GST_DEBUG_OBJECT (sink, "added new client ip %s:%u with socket %p",
-          ip, g_inet_socket_address_get_port (addr), client_socket);
+    GST_DEBUG_OBJECT (sink, "added new client ip %s:%u with socket %p",
+        ip, g_inet_socket_address_get_port (addr), client_socket);
 
-      g_free (ip);
-      g_object_unref (addr);
-    } else {
-      /* This can happen when the client immediately closes the connection */
-      GST_DEBUG_OBJECT (sink, "added new client (no address) with socket %p",
-          client_socket);
-    }
+    g_free (ip);
   }
 #endif
 
-  g_object_unref (client_socket);
   return TRUE;
 
   /* ERRORS */
@@ -254,7 +240,7 @@ gst_tcp_server_sink_set_property (GObject * object, guint prop_id,
         break;
       }
       g_free (sink->host);
-      sink->host = g_value_dup_string (value);
+      sink->host = g_strdup (g_value_get_string (value));
       break;
     case PROP_PORT:
       sink->server_port = g_value_get_int (value);
@@ -296,24 +282,43 @@ gst_tcp_server_sink_init_send (GstMultiHandleSink * parent)
 {
   GstTCPServerSink *this = GST_TCP_SERVER_SINK (parent);
   GError *err = NULL;
-  GList *addrs;
-  GList *cur_addr;
+  GInetAddress *addr;
   GSocketAddress *saddr;
+  GResolver *resolver;
   gint bound_port;
 
-  addrs =
-      tcp_get_addresses (GST_ELEMENT (this), this->host,
-      this->element.cancellable, &err);
-  if (!addrs)
-    goto name_resolve;
+  /* look up name if we need to */
+  addr = g_inet_address_new_from_string (this->host);
+  if (!addr) {
+    GList *results;
 
-  /* iterate over addresses until one works */
-  cur_addr = addrs;
+    resolver = g_resolver_get_default ();
+
+    results =
+        g_resolver_lookup_by_name (resolver, this->host,
+        this->element.cancellable, &err);
+    if (!results)
+      goto name_resolve;
+    addr = G_INET_ADDRESS (g_object_ref (results->data));
+
+    g_resolver_free_addresses (results);
+    g_object_unref (resolver);
+  }
+#ifndef GST_DISABLE_GST_DEBUG
+  {
+    gchar *ip = g_inet_address_to_string (addr);
+
+    GST_DEBUG_OBJECT (this, "IP address for host %s is %s", this->host, ip);
+    g_free (ip);
+  }
+#endif
+  saddr = g_inet_socket_address_new (addr, this->server_port);
+  g_object_unref (addr);
+
+  /* create the server listener socket */
   this->server_socket =
-      tcp_create_socket (GST_ELEMENT (this), &cur_addr, this->server_port,
-      &saddr, &err);
-  g_list_free_full (addrs, g_object_unref);
-
+      g_socket_new (g_socket_address_get_family (saddr), G_SOCKET_TYPE_STREAM,
+      G_SOCKET_PROTOCOL_TCP, &err);
   if (!this->server_socket)
     goto no_socket;
 
@@ -368,17 +373,19 @@ no_socket:
     GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
         ("Failed to create socket: %s", err->message));
     g_clear_error (&err);
+    g_object_unref (saddr);
     return FALSE;
   }
 name_resolve:
   {
     if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-      GST_DEBUG_OBJECT (this, "Cancelled name resolution");
+      GST_DEBUG_OBJECT (this, "Cancelled name resolval");
     } else {
       GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
           ("Failed to resolve host '%s': %s", this->host, err->message));
     }
     g_clear_error (&err);
+    g_object_unref (resolver);
     return FALSE;
   }
 bind_failed:
