@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,26 +42,31 @@ SinkPipeline::SinkPipeline(QGraphicsView *parent) : QObject(parent)
     if (sret != GST_STATE_CHANGE_SUCCESS) {
       gst_element_set_state (sink, GST_STATE_NULL);
       gst_object_unref (sink);
-    }
-  } else if ((sink = gst_element_factory_make ("ximagesink", NULL))) {
-    sret = gst_element_set_state (sink, GST_STATE_READY);
-    if (sret != GST_STATE_CHANGE_SUCCESS) {
-      gst_element_set_state (sink, GST_STATE_NULL);
-      gst_object_unref (sink);
-    }
-  } else if (strcmp (DEFAULT_VIDEOSINK, "xvimagesink") != 0 &&
-             strcmp (DEFAULT_VIDEOSINK, "ximagesink") != 0) {
-    if ((sink = gst_element_factory_make (DEFAULT_VIDEOSINK, NULL))) {
-      if (!GST_IS_BIN (sink)) {
+  
+      if ((sink = gst_element_factory_make ("ximagesink", NULL))) {
         sret = gst_element_set_state (sink, GST_STATE_READY);
         if (sret != GST_STATE_CHANGE_SUCCESS) {
           gst_element_set_state (sink, GST_STATE_NULL);
           gst_object_unref (sink);
-          sink = NULL;
+    
+          if (strcmp (DEFAULT_VIDEOSINK, "xvimagesink") != 0 &&
+              strcmp (DEFAULT_VIDEOSINK, "ximagesink") != 0) {
+    
+            if ((sink = gst_element_factory_make (DEFAULT_VIDEOSINK, NULL))) {
+              if (!GST_IS_BIN (sink)) {
+                sret = gst_element_set_state (sink, GST_STATE_READY);
+                if (sret != GST_STATE_CHANGE_SUCCESS) {
+                  gst_element_set_state (sink, GST_STATE_NULL);
+                  gst_object_unref (sink);
+                  sink = NULL;
+                }
+              } else {
+                gst_object_unref (sink);
+                sink = NULL;
+              }
+            }
+          }
         }
-      } else {
-        gst_object_unref (sink);
-        sink = NULL;
       }
     }
   }
@@ -108,7 +113,7 @@ int main( int argc, char **argv )
 
     QGraphicsView view( &scene );
     view.resize(320, 240);
-    view.setWindowTitle("GstVideoOverlay Qt GraphicsView demo");
+    view.setWindowTitle("GstXOverlay Qt GraphicsView demo");
     view.show();
 
     gst_init (&argc, &argv);
