@@ -17,13 +17,16 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GTK versions (>= 3.3.0) */
+#define GDK_DISABLE_DEPRECATION_WARNINGS
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -42,7 +45,7 @@
 #include <linux/input.h>
 #include <linux/joystick.h>
 
-#ifdef HAVE_X11
+#ifdef HAVE_X
 #include <gdk/gdkx.h>
 #endif
 #include <gst/video/videooverlay.h>
@@ -604,7 +607,7 @@ make_avi_msmpeg4v3_mp3_pipeline (const gchar * location)
 
   audio_bin = gst_bin_new ("a_decoder_bin");
   a_queue = gst_element_factory_make_or_warn ("queue", "a_queue");
-  a_decoder = gst_element_factory_make_or_warn ("mpg123audiodec", "a_dec");
+  a_decoder = gst_element_factory_make_or_warn ("mad", "a_dec");
   a_convert = gst_element_factory_make_or_warn ("audioconvert", "a_convert");
   audiosink = gst_element_factory_make_or_warn (ASINK, "a_sink");
 
@@ -668,8 +671,8 @@ make_mp3_pipeline (const gchar * location)
   pipeline = gst_pipeline_new ("app");
 
   src = gst_element_factory_make_or_warn (SOURCE, "src");
-  parser = gst_element_factory_make_or_warn ("mpegaudioparse", "parse");
-  decoder = gst_element_factory_make_or_warn ("mpg123audiodec", "dec");
+  parser = gst_element_factory_make_or_warn ("mp3parse", "parse");
+  decoder = gst_element_factory_make_or_warn ("mad", "dec");
   queue = gst_element_factory_make_or_warn ("queue", "queue");
   audiosink = gst_element_factory_make_or_warn (ASINK, "sink");
 
@@ -719,7 +722,7 @@ make_avi_pipeline (const gchar * location)
   gst_element_link (src, demux);
 
   audio_bin = gst_bin_new ("a_decoder_bin");
-  a_decoder = gst_element_factory_make_or_warn ("mpg123audiodec", "a_dec");
+  a_decoder = gst_element_factory_make_or_warn ("mad", "a_dec");
   audiosink = gst_element_factory_make_or_warn (ASINK, "a_sink");
   a_queue = gst_element_factory_make_or_warn ("queue", "a_queue");
   gst_element_link (a_decoder, a_queue);
@@ -779,14 +782,15 @@ make_mpeg_pipeline (const gchar * location)
   src = gst_element_factory_make_or_warn (SOURCE, "src");
   g_object_set (G_OBJECT (src), "location", location, NULL);
 
-  demux = gst_element_factory_make_or_warn ("mpegdemux", "demux");
+  //demux = gst_element_factory_make_or_warn ("mpegdemux", "demux");
+  demux = gst_element_factory_make_or_warn ("flupsdemux", "demux");
 
   gst_bin_add (GST_BIN (pipeline), src);
   gst_bin_add (GST_BIN (pipeline), demux);
   gst_element_link (src, demux);
 
   audio_bin = gst_bin_new ("a_decoder_bin");
-  a_decoder = gst_element_factory_make_or_warn ("mpg123audiodec", "a_dec");
+  a_decoder = gst_element_factory_make_or_warn ("mad", "a_dec");
   a_queue = gst_element_factory_make_or_warn ("queue", "a_queue");
   audiosink = gst_element_factory_make_or_warn (ASINK, "a_sink");
   gst_bin_add (GST_BIN (audio_bin), a_decoder);
@@ -863,7 +867,7 @@ make_mpegnt_pipeline (const gchar * location)
   gst_element_link (src, demux);
 
   audio_bin = gst_bin_new ("a_decoder_bin");
-  a_decoder = gst_element_factory_make_or_warn ("mpg123audiodec", "a_dec");
+  a_decoder = gst_element_factory_make_or_warn ("mad", "a_dec");
   a_queue = gst_element_factory_make_or_warn ("queue", "a_queue");
   audiosink = gst_element_factory_make_or_warn (ASINK, "a_sink");
   //g_object_set (G_OBJECT (audiosink), "fragment", 0x00180008, NULL);
@@ -960,8 +964,9 @@ static GstElement *
 make_parselaunch_pipeline (const gchar * description)
 {
   GstElement *pipeline;
+  GError *error = NULL;
 
-  pipeline = gst_parse_launch (description, NULL);
+  pipeline = gst_parse_launch (description, &error);
 
   seekable_elements = g_list_prepend (seekable_elements, pipeline);
 
@@ -1812,7 +1817,6 @@ update_streams (GstPipeline * pipeline)
         str = gst_tag_list_to_string (tags);
         g_print ("video %d: %s\n", i, str);
         g_free (str);
-        gst_tag_list_unref (tags);
       }
       /* find good name for the label */
       name = g_strdup_printf ("video %d", i + 1);
@@ -1830,7 +1834,6 @@ update_streams (GstPipeline * pipeline)
         str = gst_tag_list_to_string (tags);
         g_print ("audio %d: %s\n", i, str);
         g_free (str);
-        gst_tag_list_unref (tags);
       }
       /* find good name for the label */
       name = g_strdup_printf ("audio %d", i + 1);
@@ -1858,7 +1861,6 @@ update_streams (GstPipeline * pipeline)
         if (value && G_VALUE_HOLDS_STRING (value)) {
           name = g_strdup_printf ("text %s", g_value_get_string (value));
         }
-        gst_tag_list_unref (tags);
       }
       /* find good name for the label if we didn't use a tag */
       if (name == NULL)
@@ -1945,7 +1947,6 @@ init_visualization_features (void)
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (vis_combo), name);
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (vis_combo), 0);
-  gst_plugin_feature_list_free (list);
 }
 
 static void
@@ -2024,7 +2025,7 @@ shot_cb (GtkButton * button, gpointer data)
 
     /* get the snapshot buffer format now. We set the caps on the appsink so
      * that it can only be an rgb buffer. The only thing we have not specified
-     * on the caps is the height, which is dependent on the pixel-aspect-ratio
+     * on the caps is the height, which is dependant on the pixel-aspect-ratio
      * of the source material */
 #if 0
     caps = GST_BUFFER_CAPS (buffer);
@@ -2055,7 +2056,9 @@ shot_cb (GtkButton * button, gpointer data)
     /* save the pixbuf */
     gdk_pixbuf_save (pixbuf, "snapshot.png", "png", &error, NULL);
     gst_buffer_unmap (buffer, &map);
-    g_clear_error (&error);
+
+    /* save the pixbuf */
+    gdk_pixbuf_save (pixbuf, "snapshot.png", "png", &error, NULL);
 
   done:
     gst_buffer_unref (buffer);
@@ -2349,7 +2352,7 @@ do_stream_buffering (gint percent)
     }
   } else {
     /* buffering busy */
-    if (!buffering && state == GST_STATE_PLAYING) {
+    if (buffering == FALSE && state == GST_STATE_PLAYING) {
       /* we were not buffering but PLAYING, PAUSE  the pipeline. */
       if (!is_live) {
         fprintf (stderr, "Buffering, setting pipeline to PAUSED ...\n");
@@ -2416,7 +2419,7 @@ msg_clock_lost (GstBus * bus, GstMessage * message, GstPipeline * data)
   gst_element_set_state (pipeline, GST_STATE_PLAYING);
 }
 
-#ifdef HAVE_X11
+#ifdef HAVE_X
 
 static gulong embed_xid = 0;
 
@@ -2473,9 +2476,9 @@ realize_cb (GtkWidget * widget, gpointer data)
   /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
    * as well */
   if (!gdk_window_ensure_native (window))
-    g_error ("Couldn't create native window needed for GstVideoOverlay!");
+    g_error ("Couldn't create native window needed for GstXOverlay!");
 
-#ifdef HAVE_X11
+#ifdef HAVE_X
   embed_xid = GDK_WINDOW_XID (window);
   g_print ("Window realize: video window XID = %lu\n", embed_xid);
 #endif
@@ -2509,7 +2512,7 @@ connect_bus_signals (GstElement * pipeline)
 {
   GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
 
-#ifdef HAVE_X11
+#ifdef HAVE_X
   /* handle prepare-window-handle element message synchronously */
   gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler,
       pipeline, NULL);
@@ -2552,10 +2555,6 @@ connect_bus_signals (GstElement * pipeline)
   gst_object_unref (bus);
 }
 
-#if !GLIB_CHECK_VERSION(2,70,0)
-#define g_pattern_spec_match_string g_pattern_match_string
-#endif
-
 /* Return GList of paths described in location string */
 static GList *
 handle_wildcards (const gchar * location)
@@ -2575,7 +2574,7 @@ handle_wildcards (const gchar * location)
   }
 
   while ((name = g_dir_read_name (dir)) != NULL) {
-    if (g_pattern_spec_match_string (pspec, name)) {
+    if (g_pattern_match_string (pspec, name)) {
       res = g_list_append (res, g_strjoin ("/", path, name, NULL));
       g_print ("  found clip %s\n", name);
     }
@@ -2624,7 +2623,7 @@ read_joystick (GIOChannel * source, GIOCondition condition, gpointer user_data)
       &bytes_read, &err);
   if (err) {
     g_print ("error reading from joystick: %s", err->message);
-    g_clear_error (&err);
+    g_error_free (err);
     return FALSE;
   } else if (bytes_read != sizeof (struct js_event)) {
     g_print ("error reading joystick, read %u bytes of %u\n",
@@ -2652,7 +2651,7 @@ int
 main (int argc, char **argv)
 {
   GtkWidget *window, *hbox, *vbox, *panel, *expander, *pb2vbox, *boxes,
-      *flaggrid, *boxes2, *step;
+      *flagtable, *boxes2, *step;
   GtkWidget *play_button, *pause_button, *stop_button, *shot_button;
   GtkWidget *accurate_checkbox, *key_checkbox, *loop_checkbox, *flush_checkbox;
   GtkWidget *scrub_checkbox, *play_scrub_checkbox;
@@ -2678,11 +2677,9 @@ main (int argc, char **argv)
 
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
     g_print ("Error initializing: %s\n", err->message);
-    g_option_context_free (ctx);
-    g_clear_error (&err);
     exit (1);
   }
-  g_option_context_free (ctx);
+
   GST_DEBUG_CATEGORY_INIT (seek_debug, "seek", 0, "seek example");
 
   if (argc != 3) {
@@ -2730,22 +2727,20 @@ main (int argc, char **argv)
   video_window = gtk_drawing_area_new ();
   g_signal_connect (video_window, "draw", G_CALLBACK (draw_cb), NULL);
   g_signal_connect (video_window, "realize", G_CALLBACK (realize_cb), NULL);
+  gtk_widget_set_double_buffered (video_window, FALSE);
 
   statusbar = gtk_statusbar_new ();
   status_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "seek");
   gtk_statusbar_push (GTK_STATUSBAR (statusbar), status_id, "Stopped");
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-  flaggrid = gtk_grid_new ();
+  hbox = gtk_hbox_new (FALSE, 0);
+  vbox = gtk_vbox_new (FALSE, 0);
+  flagtable = gtk_table_new (4, 2, FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
 
   /* media controls */
-  play_button = gtk_button_new_from_icon_name ("media-playback-start",
-      GTK_ICON_SIZE_BUTTON);
-  pause_button = gtk_button_new_from_icon_name ("media-playback-pause",
-      GTK_ICON_SIZE_BUTTON);
-  stop_button = gtk_button_new_from_icon_name ("media-playback-stop",
-      GTK_ICON_SIZE_BUTTON);
+  play_button = gtk_button_new_from_stock (GTK_STOCK_MEDIA_PLAY);
+  pause_button = gtk_button_new_from_stock (GTK_STOCK_MEDIA_PAUSE);
+  stop_button = gtk_button_new_from_stock (GTK_STOCK_MEDIA_STOP);
 
   /* seek flags */
   accurate_checkbox = gtk_check_button_new_with_label ("Accurate Seek");
@@ -2782,7 +2777,7 @@ main (int argc, char **argv)
     GtkWidget *hbox;
 
     step = gtk_expander_new ("step options");
-    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    hbox = gtk_hbox_new (FALSE, 0);
 
     format_combo = gtk_combo_box_text_new ();
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format_combo),
@@ -2803,8 +2798,7 @@ main (int argc, char **argv)
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (step_rate_spinbutton), 1.0);
     gtk_box_pack_start (GTK_BOX (hbox), step_rate_spinbutton, FALSE, FALSE, 2);
 
-    step_button = gtk_button_new_from_icon_name ("media-seek-forward",
-        GTK_ICON_SIZE_BUTTON);
+    step_button = gtk_button_new_from_stock (GTK_STOCK_MEDIA_FORWARD);
     gtk_button_set_label (GTK_BUTTON (step_button), "Step");
     gtk_box_pack_start (GTK_BOX (hbox), step_button, FALSE, FALSE, 2);
 
@@ -2820,8 +2814,7 @@ main (int argc, char **argv)
 
     shuttle_adjustment =
         GTK_ADJUSTMENT (gtk_adjustment_new (0.0, -3.00, 4.0, 0.1, 1.0, 1.0));
-    shuttle_hscale = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL,
-        shuttle_adjustment);
+    shuttle_hscale = gtk_hscale_new (shuttle_adjustment);
     gtk_scale_set_digits (GTK_SCALE (shuttle_hscale), 2);
     gtk_scale_set_value_pos (GTK_SCALE (shuttle_hscale), GTK_POS_TOP);
     g_signal_connect (shuttle_hscale, "value_changed",
@@ -2837,7 +2830,7 @@ main (int argc, char **argv)
   /* seek bar */
   adjustment =
       GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, 100.0, 0.1, 1.0, 1.0));
-  hscale = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, adjustment);
+  hscale = gtk_hscale_new (adjustment);
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
   gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_RIGHT);
   gtk_range_set_show_fill_level (GTK_RANGE (hscale), TRUE);
@@ -2852,7 +2845,7 @@ main (int argc, char **argv)
 
   if (pipeline_type == 16) {
     /* the playbin panel controls for the video/audio/subtitle tracks */
-    panel = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    panel = gtk_hbox_new (FALSE, 0);
     video_combo = gtk_combo_box_text_new ();
     audio_combo = gtk_combo_box_text_new ();
     text_combo = gtk_combo_box_text_new ();
@@ -2869,7 +2862,7 @@ main (int argc, char **argv)
     g_signal_connect (G_OBJECT (text_combo), "changed",
         G_CALLBACK (text_combo_cb), pipeline);
     /* playbin panel for flag checkboxes and volume/mute */
-    boxes = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    boxes = gtk_hbox_new (FALSE, 0);
     vis_checkbox = gtk_check_button_new_with_label ("Vis");
     video_checkbox = gtk_check_button_new_with_label ("Video");
     audio_checkbox = gtk_check_button_new_with_label ("Audio");
@@ -2913,9 +2906,8 @@ main (int argc, char **argv)
     g_signal_connect (G_OBJECT (volume_spinbutton), "value_changed",
         G_CALLBACK (volume_spinbutton_changed_cb), pipeline);
     /* playbin panel for snapshot */
-    boxes2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    shot_button = gtk_button_new_from_icon_name ("document-save",
-        GTK_ICON_SIZE_BUTTON);
+    boxes2 = gtk_hbox_new (FALSE, 0);
+    shot_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
     gtk_widget_set_tooltip_text (shot_button,
         "save a screenshot .png in the current directory");
     g_signal_connect (G_OBJECT (shot_button), "clicked", G_CALLBACK (shot_cb),
@@ -2944,20 +2936,22 @@ main (int argc, char **argv)
   gtk_box_pack_start (GTK_BOX (hbox), play_button, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), pause_button, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), stop_button, FALSE, FALSE, 2);
-  gtk_box_pack_start (GTK_BOX (hbox), flaggrid, FALSE, FALSE, 2);
-  gtk_grid_attach (GTK_GRID (flaggrid), accurate_checkbox, 0, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), flush_checkbox, 1, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), loop_checkbox, 2, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), key_checkbox, 0, 1, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), scrub_checkbox, 1, 1, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), play_scrub_checkbox, 2, 1, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), skip_checkbox, 3, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), rate_label, 4, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (flaggrid), rate_spinbutton, 4, 1, 1, 1);
-
+  gtk_box_pack_start (GTK_BOX (hbox), flagtable, FALSE, FALSE, 2);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), accurate_checkbox, 0, 1, 0,
+      1);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), flush_checkbox, 1, 2, 0, 1);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), loop_checkbox, 2, 3, 0, 1);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), key_checkbox, 0, 1, 1, 2);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), scrub_checkbox, 1, 2, 1, 2);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), play_scrub_checkbox, 2, 3,
+      1, 2);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), skip_checkbox, 3, 4, 0, 1);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), rate_label, 4, 5, 0, 1);
+  gtk_table_attach_defaults (GTK_TABLE (flagtable), rate_spinbutton, 4, 5, 1,
+      2);
   if (panel && boxes && boxes2) {
     expander = gtk_expander_new ("playbin options");
-    pb2vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+    pb2vbox = gtk_vbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (pb2vbox), panel, FALSE, FALSE, 2);
     gtk_box_pack_start (GTK_BOX (pb2vbox), boxes, FALSE, FALSE, 2);
     gtk_box_pack_start (GTK_BOX (pb2vbox), boxes2, FALSE, FALSE, 2);
@@ -3002,10 +2996,8 @@ main (int argc, char **argv)
    * asks for the XID of the window to render onto */
   gtk_widget_realize (window);
 
-#ifdef HAVE_X11
   /* we should have the XID now */
   g_assert (embed_xid != 0);
-#endif
 
   if (verbose) {
     g_signal_connect (pipeline, "deep_notify",
@@ -3025,7 +3017,6 @@ main (int argc, char **argv)
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
   g_print ("free pipeline\n");
-  g_array_free (vis_entries, TRUE);
   gst_object_unref (pipeline);
 
   g_list_foreach (paths, (GFunc) g_free, NULL);
